@@ -66,6 +66,8 @@ int RUNMAIN(arcsub)(int argc, char **argv) {
     logger.msg(Arc::ERROR, "Failed configuration initialization");
     return 1;
   }
+  if (opt.force_system_ca) usercfg.CAUseSystem(true);
+  if (opt.force_grid_ca) usercfg.CAUseSystem(false);
 
   if (opt.show_plugins) {
     std::list<std::string> types;
@@ -167,58 +169,51 @@ int RUNMAIN(arcsub)(int argc, char **argv) {
     }
   }
 
-  // Check if X.509 credentials are needed for data staging
-  bool need_proxy = false;
-  for (std::list<Arc::JobDescription>::iterator jobIt = jobdescriptionlist.begin(); jobIt != jobdescriptionlist.end(); ++jobIt) {
-    if(jobneedsproxy(*jobIt)) {
-      need_proxy = true;
+  DelegationType delegation_type = UndefinedDelegation;
+  if(!opt.getDelegationType(logger, usercfg, delegation_type))
+    return 1;
+  AuthenticationType authentication_type = UndefinedAuthentication;
+  if(!opt.getAuthenticationType(logger, usercfg, authentication_type))
+    return 1;
+  switch(authentication_type) {
+    case NoAuthentication:
+      usercfg.CommunicationAuthType(Arc::UserConfig::AuthTypeNone);
+      break;
+    case X509Authentication:
+      usercfg.CommunicationAuthType(Arc::UserConfig::AuthTypeCert);
+      break;
+    case TokenAuthentication:
+      usercfg.CommunicationAuthType(Arc::UserConfig::AuthTypeToken);
+      break;
+    case UndefinedAuthentication:
+    default:
+      usercfg.CommunicationAuthType(Arc::UserConfig::AuthTypeUndefined);
       break;
-    }
-  }
-  if(need_proxy || usercfg.OToken().empty()) {
-    if (!checkproxy(usercfg)) {
-      return 1;
-    }
   }
 
-  if ( opt.isARC6TargetSelectionOptions(logger) ) {
-    // canonicalize endpoint types
-    if (!opt.canonicalizeARC6InterfaceTypes(logger)) return 1;
+  // canonicalize endpoint types
+  if (!opt.canonicalizeARCInterfaceTypes(logger)) return 1;
 
-    // get endpoint batches according to ARC6 target selection logic
-    std::list<std::list<Arc::Endpoint> > endpoint_batches;
-    bool info_discovery = prepare_submission_endpoint_batches(usercfg, opt, endpoint_batches);
-  
-    // add rejectdiscovery if defined
-    if (!opt.rejectdiscovery.empty()) usercfg.AddRejectDiscoveryURLs(opt.rejectdiscovery);
+  // get endpoint batches
+  std::list<std::list<Arc::Endpoint> > endpoint_batches;
+  bool info_discovery = prepare_submission_endpoint_batches(usercfg, opt, endpoint_batches);
 
-    // action: dumpjobdescription
-    if (opt.dumpdescription) {
-        if (!info_discovery) {
-          logger.msg(Arc::ERROR,"Cannot adapt job description to the submission target when information discovery is turned off");
-          return 1;
-        }
-        // dump description only for priority submission interface, no fallbacks
-        std::list<Arc::Endpoint> services = endpoint_batches.front();
-        std::string req_sub_iface;
-        if (!opt.submit_types.empty()) req_sub_iface = opt.submit_types.front();
-        return dumpjobdescription(usercfg, jobdescriptionlist, services, req_sub_iface);
-    }
+  // add rejectdiscovery if defined
+  if (!opt.rejectdiscovery.empty()) usercfg.AddRejectDiscoveryURLs(opt.rejectdiscovery);
 
-    // default action: start submission cycle
-    return submit_jobs(usercfg, endpoint_batches, info_discovery, opt.jobidoutfile, jobdescriptionlist);
-  } else {
-    // Legacy target selection submission logic
-    std::list<Arc::Endpoint> services = getServicesFromUserConfigAndCommandLine(usercfg, opt.indexurls, opt.clusters, opt.requestedSubmissionInterfaceName, opt.infointerface);
-
-    if (!opt.direct_submission) {
-      usercfg.AddRejectDiscoveryURLs(opt.rejectdiscovery);
-    }
-
-    if (opt.dumpdescription) {
-      return dumpjobdescription(usercfg, jobdescriptionlist, services, opt.requestedSubmissionInterfaceName);
-    }
-
-    return legacy_submit(usercfg, jobdescriptionlist, services, opt.requestedSubmissionInterfaceName, opt.jobidoutfile, opt.direct_submission);
+  // action: dumpjobdescription
+  if (opt.dumpdescription) {
+      if (!info_discovery) {
+        logger.msg(Arc::ERROR,"Cannot adapt job description to the submission target when information discovery is turned off");
+        return 1;
+      }
+      // dump description only for priority submission interface, no fallbacks
+      std::list<Arc::Endpoint> services = endpoint_batches.front();
+      std::string req_sub_iface;
+      if (!opt.submit_types.empty()) req_sub_iface = opt.submit_types.front();
+      return dumpjobdescription(usercfg, jobdescriptionlist, services, req_sub_iface);
   }
+
+  // default action: start submission cycle
+  return submit_jobs(usercfg, endpoint_batches, info_discovery, opt.jobidoutfile, jobdescriptionlist, delegation_type, opt.instances_min, opt.instances_max);
 }
