@@ -22,6 +22,8 @@
 #include <arc/loader/FinderLoader.h>
 #include <arc/OptionParser.h>
 
+#include "utils.h"
+
 static Arc::Logger logger(Arc::Logger::getRootLogger(), "arcls");
 
 
@@ -145,20 +147,7 @@ static bool arcls(const Arc::URL& dir_url,
     return false;
   }
   if (url->RequiresCredentials()) {
-    if (!usercfg.InitializeCredentials(Arc::initializeCredentialsType::RequireCredentials)) {
-      logger.msg(Arc::ERROR, "Unable to list content of %s", dir_url.str());
-      logger.msg(Arc::ERROR, "Invalid credentials, please check proxy and/or CA certificates");
-      return false;
-    }
-    Arc::Credential holder(usercfg);
-    if (!holder.IsValid()) {
-      if (holder.GetEndTime() < Arc::Time()) {
-        logger.msg(Arc::ERROR, "Proxy expired");
-      }
-      logger.msg(Arc::ERROR, "Unable to list content of %s", dir_url.str());
-      logger.msg(Arc::ERROR, "Invalid credentials, please check proxy and/or CA certificates");
-      return false;
-    }
+    if(!initProxy(logger, usercfg, dir_url)) return false;
   }
   url->SetSecure(false);
 
@@ -332,6 +321,36 @@ static int runmain(int argc, char **argv) {
                     istring("configuration file (default ~/.arc/client.conf)"),
                     istring("filename"), conffile);
 
+  bool no_authentication = false;
+  options.AddOption('\0', "no-authentication",
+              istring("do not perform any authentication for opened connections"),
+              no_authentication);
+
+  bool x509_authentication = false;
+  options.AddOption('\0', "x509-authentication",
+              istring("perform X.509 authentication for opened connections"),
+              x509_authentication);
+
+  bool token_authentication = false;
+  options.AddOption('\0', "token-authentication",
+              istring("perform token authentication for opened connections"),
+              token_authentication);
+
+  bool force_system_ca = false;
+  options.AddOption('\0', "systemca",
+              istring("force using CA certificates configuration provided by OpenSSL"),
+              force_system_ca);
+
+  bool force_grid_ca = false;
+  options.AddOption('\0', "gridca",
+              istring("force using CA certificates configuration for Grid services (typically IGTF)"),
+              force_grid_ca);
+    
+  bool allow_insecure_connection = false;
+  options.AddOption('\0', "allowinsecureconnection",
+              istring("allow TLS connection which failed verification"),
+              allow_insecure_connection);
+
   std::string debug;
   options.AddOption('d', "debug",
                     istring("FATAL, ERROR, WARNING, INFO, VERBOSE or DEBUG"),
@@ -378,6 +397,28 @@ static int runmain(int argc, char **argv) {
     return 1;
   }
   usercfg.UtilsDirPath(Arc::UserConfig::ARCUSERDIRECTORY());
+  if (force_system_ca) usercfg.CAUseSystem(true);
+  if (force_grid_ca) usercfg.CAUseSystem(false);
+  if (allow_insecure_connection) usercfg.TLSAllowInsecure(true);
+ 
+  AuthenticationType authentication_type = UndefinedAuthentication;
+  if(!getAuthenticationType(logger, usercfg, no_authentication, x509_authentication, token_authentication, authentication_type))
+    return 1;
+  switch(authentication_type) {
+    case NoAuthentication:
+      usercfg.CommunicationAuthType(Arc::UserConfig::AuthTypeNone);
+      break;
+    case X509Authentication:
+      usercfg.CommunicationAuthType(Arc::UserConfig::AuthTypeCert);
+      break;
+    case TokenAuthentication:
+      usercfg.CommunicationAuthType(Arc::UserConfig::AuthTypeToken);
+      break;
+    case UndefinedAuthentication:
+    default:
+      usercfg.CommunicationAuthType(Arc::UserConfig::AuthTypeUndefined);
+      break;
+  }
 
   if (debug.empty() && !usercfg.Verbosity().empty())
     Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(usercfg.Verbosity()));
