@@ -5,7 +5,7 @@
 
 #include <Python.h>
 #include <arc/UserConfig.h>
-#include <arc/client/Broker.h>
+#include <arc/compute/Broker.h>
 
 namespace Arc {
 
@@ -17,7 +17,7 @@ namespace Arc {
     
     bool operator()(const ExecutionTarget&, const ExecutionTarget&) const;
     bool match(const ExecutionTarget&) const;
-    void set(const JobDescription&);
+    void set(const JobDescription&) const;
 
   private:
     PyObject *arc_module;
@@ -27,6 +27,7 @@ namespace Arc {
     PyObject *module;
     PyObject *klass;
     PyObject *object;
+    bool valid;
 
     static Logger logger;
     static PyThreadState *tstate;
