@@ -92,7 +92,8 @@ namespace Arc {
   }
 
 
-  BaseConfig::BaseConfig() : plugin_paths(ArcLocation::GetPlugins()) {}
+  BaseConfig::BaseConfig() : plugin_paths(ArcLocation::GetPlugins()), systemca(false), tlsallowinsecure(false),
+                             otoken_for_auth(true), cert_for_auth(true) {}
 
   void BaseConfig::AddPluginsPath(const std::string& path) {
     plugin_paths.push_back(path);
@@ -134,6 +135,14 @@ namespace Arc {
     otoken = token;
   }
 
+  void BaseConfig::SetSystemCA(bool use_default) {
+    systemca = use_default;
+  }
+
+  void BaseConfig::SetTLSAllowInsecure(bool allow_insecure) {
+    tlsallowinsecure = allow_insecure;
+  }
+
   void BaseConfig::AddOverlay(XMLNode cfg) {
     overlay.Destroy();
     cfg.New(overlay);
