@@ -38,13 +38,14 @@ namespace Arc {
      public:
       StringData();
       virtual ~StringData();
-      void Assign(std::string& str);
+      void Assign(std::string& str, int content_max_size = 0);
       virtual void Append(char const* data, unsigned int size);
       virtual void Remove(unsigned int size);
       virtual char const* Get() const;
       virtual unsigned int Size() const;
      private:
       std::string* content_;
+      int content_max_size_;
     };
 
     // working directory
@@ -72,6 +73,7 @@ namespace Arc {
     std::list<std::string> envx_;
     void (*initializer_func_)(void*);
     void *initializer_arg_;
+    bool initializer_is_complex_;
     void (*kicker_func_)(void*);
     void *kicker_arg_;
     // IO handlers are called when data can be sent/received
@@ -164,12 +166,12 @@ namespace Arc {
     /// Associate stdout pipe of executable with string.
     /** This method must be called before Start(). str object
         must be valid as long as this object exists. */
-    void AssignStdout(std::string& str);
+    void AssignStdout(std::string& str, int max_size = 102400);
     void AssignStdout(Data& str);
     /// Associate stderr pipe of executable with string.
     /** This method must be called before Start(). str object
         must be valid as long as this object exists. */
-    void AssignStderr(std::string& str);
+    void AssignStderr(std::string& str, int max_size = 102400);
     void AssignStderr(Data& str);
     /// Associate stdin pipe of executable with string.
     /** This method must be called before Start(). str object
@@ -189,7 +191,7 @@ namespace Arc {
     /// Closes pipe associated with stdin handle.
     void CloseStdin(void);
     /// Assign a function to be called just after process is forked but before execution starts.
-    void AssignInitializer(void (*initializer_func)(void*), void *initializer_arg);
+    void AssignInitializer(void (*initializer_func)(void*), void *initializer_arg, bool is_complex);
     /// Assign a function to be called just after execution ends. It is executed asynchronously.
     void AssignKicker(void (*kicker_func)(void*), void *kicker_arg);
     /// Assign working directory of the process to run.
