@@ -14,6 +14,8 @@
 #define HTTP_DEFAULT_PORT 80
 #define HTTPS_DEFAULT_PORT 443
 #define HTTPG_DEFAULT_PORT 8443
+#define DAV_DEFAULT_PORT 80
+#define DAVS_DEFAULT_PORT 443
 #define LDAP_DEFAULT_PORT 389
 #define FTP_DEFAULT_PORT 21
 #define GSIFTP_DEFAULT_PORT 2811
