@@ -250,7 +250,7 @@ namespace Arc {
   }
 
   UserConfig::UserConfig(initializeCredentialsType initializeCredentials)
-    : timeout(0), keySize(0), ok(false), initializeCredentials(initializeCredentials) {
+    : timeout(0), keySize(0), ok(false), caUseSystem(false), tlsAllowInsecure(false), initializeCredentials(initializeCredentials), authType(AuthTypeUndefined) {
     if (!InitializeCredentials(initializeCredentials)) {
       return;
     }
@@ -263,7 +263,7 @@ namespace Arc {
   UserConfig::UserConfig(const std::string& conffile,
                          initializeCredentialsType initializeCredentials,
                          bool loadSysConfig)
-    : timeout(0), keySize(0), ok(false), initializeCredentials(initializeCredentials)  {
+    : timeout(0), keySize(0), ok(false), caUseSystem(false), tlsAllowInsecure(false), initializeCredentials(initializeCredentials), authType(AuthTypeUndefined)  {
     setDefaults();
     if (loadSysConfig) {
       if (Glib::file_test(SYSCONFIG(), Glib::FILE_TEST_IS_REGULAR)) {
@@ -309,7 +309,7 @@ namespace Arc {
 
   UserConfig::UserConfig(const std::string& conffile, const std::string& jfile,
                          initializeCredentialsType initializeCredentials, bool loadSysConfig)
-    : timeout(0), keySize(0), ok(false), initializeCredentials(initializeCredentials)  {
+    : timeout(0), keySize(0), ok(false), caUseSystem(false), tlsAllowInsecure(false), initializeCredentials(initializeCredentials), authType(AuthTypeUndefined)  {
     // If job list file have been specified, try to initialize it, and
     // if it fails then this object is non-valid (ok = false).
     setDefaults();
@@ -378,14 +378,38 @@ namespace Arc {
         ccfg.AddPrivateKey(keyPath);
       }
     }
-    ccfg.AddCADir(caCertificatesDirectory);
+    ccfg.SetSystemCA(caUseSystem);
+    ccfg.SetTLSAllowInsecure(tlsAllowInsecure);
+    if(!caUseSystem)
+      ccfg.AddCADir(caCertificatesDirectory);
+
+    switch(authType) {
+      case AuthTypeNone:
+         ccfg.cert_for_auth = false;
+         ccfg.otoken_for_auth = false;
+         break;
+      case AuthTypeCert:
+         ccfg.cert_for_auth = true;
+         ccfg.otoken_for_auth = false;
+         break;
+      case AuthTypeToken:
+         ccfg.cert_for_auth = false;
+         ccfg.otoken_for_auth = true;
+         break;
+      case AuthTypeUndefined:
+      default:
+         ccfg.cert_for_auth = true;
+         ccfg.otoken_for_auth = true;
+         break;
+    }
 
     if(!otoken.empty()) {
       ccfg.AddOToken(otoken);
     }
 
-    if(!overlayfile.empty())
+    if(!overlayfile.empty()) {
       ccfg.GetOverlay(overlayfile);
+    }
   }
 
   bool UserConfig::Timeout(int newTimeout) {
@@ -415,9 +439,9 @@ namespace Arc {
   }
 
   bool UserConfig::JobListType(const std::string& type) {
-    if (type != "XML" && type != "BDB" && type != "SQLITE") {
-      logger.msg(WARNING, "Unsupported job list type '%s', using 'BDB'. Supported types are: BDB, SQLITE, XML.", type);
-      joblisttype = "BDB";
+    if (type != "XML" && type != "SQLITE") {
+      logger.msg(WARNING, "Unsupported job list type '%s', using 'SQLITE'. Supported types are: SQLITE, XML.", type);
+      joblisttype = "SQLITE";
       return true;
     }
 
@@ -437,7 +461,15 @@ namespace Arc {
   }
 
   bool UserConfig::InitializeCredentials(initializeCredentialsType initializeCredentials) {
+    std::string ca_policy = GetEnv("X509_CERT_POLICY");
+    if (ca_policy == "grid") {
+      caUseSystem = false;
+    } else if(ca_policy == "system") {
+      caUseSystem = true;
+    }
+
     if(initializeCredentials == initializeCredentialsType::SkipCredentials) return true;
+
     bool res = true;
     bool require =
             ((initializeCredentials == initializeCredentialsType::RequireCredentials) ||
@@ -514,7 +546,7 @@ namespace Arc {
       file_test_status fts;
       if (test && ((fts = user_file_test(certificatePath, user)) != file_test_success)) {
         certificate_file_error_report(fts,!has_proxy,certificatePath,logger);
-        if(!has_proxy) {
+        if(!has_proxy && require) {
           res = false;
         }
         certificatePath.clear();
@@ -522,7 +554,7 @@ namespace Arc {
       keyPath = key_path;
       if (test && ((fts = private_file_test(keyPath, user)) != file_test_success)) {
         key_file_error_report(fts,!has_proxy,keyPath,logger);
-        if(!has_proxy) {
+        if(!has_proxy && require) {
           res = false;
         }
         keyPath.clear();
@@ -531,14 +563,14 @@ namespace Arc {
       file_test_status fts;
       if (test && ((fts = user_file_test(certificatePath, user)) != file_test_success)) {
         certificate_file_error_report(fts,!has_proxy,certificatePath,logger);
-        if(!has_proxy) {
+        if(!has_proxy && require) {
           res = false;
         }
         certificatePath.clear();
       }
       if (test && ((fts = private_file_test(keyPath, user)) != file_test_success)) {
         key_file_error_report(fts,!has_proxy,keyPath,logger);
-        if(!has_proxy) {
+        if(!has_proxy && require) {
           res = false;
         }
         keyPath.clear();
@@ -548,7 +580,7 @@ namespace Arc {
       file_test_status fts;
       if (test && ((fts = user_file_test(certificatePath, user)) != file_test_success)) {
         certificate_file_error_report(fts,!has_proxy,certificatePath,logger);
-        if(!has_proxy) {
+        if(!has_proxy && require) {
           res = false;
         }
         certificatePath.clear();
@@ -559,7 +591,7 @@ namespace Arc {
       file_test_status fts;
       if (test && ((fts = user_file_test(cert_path, user)) != file_test_success)) {
         certificate_file_error_report(fts,!has_proxy,cert_path,logger);
-        if(!has_proxy) {
+        if(!has_proxy && require) {
           res = false;
         }
         cert_path.clear();
@@ -601,73 +633,75 @@ namespace Arc {
         tried_paths += "'" + (*it) + "'";
       }
       if (it == search_paths.end() && !has_proxy) {
-        logger.msg(WARNING,
+        logger.msg(require?ERROR:VERBOSE,
           "Certificate and key ('%s' and '%s') not found in any of the paths: %s", "usercert.pem", "userkey.pem", tried_paths);
-        logger.msg(WARNING,
+        logger.msg(require?ERROR:VERBOSE,
           "If the proxy or certificate/key does exist, you can manually specify the locations via environment variables "
           "'%s'/'%s' or '%s', or the '%s'/'%s' or '%s' attributes in the client configuration file (e.g. '%s')",
           "X509_USER_CERT", "X509_USER_KEY", "X509_USER_PROXY", "certificatepath", "proxypath", "keypath", "~/.arc/client.conf");
       }
-      if((certificatePath.empty() || keyPath.empty()) && !has_proxy) {
+      if((certificatePath.empty() || keyPath.empty()) && !has_proxy && require) {
         res = false;
       }
     }
 
     if(!noca) {
-      std::string ca_dir = GetEnv("X509_CERT_DIR");
-      //std::cerr<<"-- ca_dir = "<<ca_dir<<std::endl;
-      if (!ca_dir.empty()) {
-        caCertificatesDirectory = ca_dir;
-        if (test && !dir_test(caCertificatesDirectory)) {
-          //std::cerr<<"-- ca_dir test failed"<<std::endl;
-          if(require) {
-            logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified.", caCertificatesDirectory);
-            res = false;
+      if (!caUseSystem) {
+        std::string ca_dir = GetEnv("X509_CERT_DIR");
+        //std::cerr<<"-- ca_dir = "<<ca_dir<<std::endl;
+        if (!ca_dir.empty()) {
+          caCertificatesDirectory = ca_dir;
+          if (test && !dir_test(caCertificatesDirectory)) {
+            //std::cerr<<"-- ca_dir test failed"<<std::endl;
+            if(require) {
+              logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified.", caCertificatesDirectory);
+              res = false;
+            }
+            caCertificatesDirectory.clear();
           }
-          caCertificatesDirectory.clear();
-        }
-      } else if (!caCertificatesDirectory.empty()) {
-        //std::cerr<<"-- caCertificatesDirectory = "<<caCertificatesDirectory<<std::endl;
-        if (test && !dir_test(caCertificatesDirectory)) {
-          //std::cerr<<"-- caCertificatesDirectory test failed"<<std::endl;
-          if(require) {
-            logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified.", caCertificatesDirectory);
-            res = false;
+        } else if (!caCertificatesDirectory.empty()) {
+          //std::cerr<<"-- caCertificatesDirectory = "<<caCertificatesDirectory<<std::endl;
+          if (test && !dir_test(caCertificatesDirectory)) {
+            //std::cerr<<"-- caCertificatesDirectory test failed"<<std::endl;
+            if(require) {
+              logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified.", caCertificatesDirectory);
+              res = false;
+            }
+            caCertificatesDirectory.clear();
           }
-          caCertificatesDirectory.clear();
-        }
-      } else {
-        caCertificatesDirectory = home_path+G_DIR_SEPARATOR_S+".arc"+G_DIR_SEPARATOR_S+"certificates";
-        //std::cerr<<"-- option 1 = "<<caCertificatesDirectory<<std::endl;
-        if (test && !dir_test(caCertificatesDirectory)) {
-          caCertificatesDirectory = home_path+G_DIR_SEPARATOR_S+".globus"+G_DIR_SEPARATOR_S+"certificates";
-          //std::cerr<<"-- option 2 = "<<caCertificatesDirectory<<std::endl;
-          if (!dir_test(caCertificatesDirectory)) {
-            caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"etc"+G_DIR_SEPARATOR_S+"certificates";
-            //std::cerr<<"-- option 3 = "<<caCertificatesDirectory<<std::endl;
+        } else {
+          caCertificatesDirectory = home_path+G_DIR_SEPARATOR_S+".arc"+G_DIR_SEPARATOR_S+"certificates";
+          //std::cerr<<"-- option 1 = "<<caCertificatesDirectory<<std::endl;
+          if (test && !dir_test(caCertificatesDirectory)) {
+            caCertificatesDirectory = home_path+G_DIR_SEPARATOR_S+".globus"+G_DIR_SEPARATOR_S+"certificates";
+            //std::cerr<<"-- option 2 = "<<caCertificatesDirectory<<std::endl;
             if (!dir_test(caCertificatesDirectory)) {
-              caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"etc"+G_DIR_SEPARATOR_S+"grid-security"+G_DIR_SEPARATOR_S+"certificates";
-              //std::cerr<<"-- option 4 = "<<caCertificatesDirectory<<std::endl;
+              caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"etc"+G_DIR_SEPARATOR_S+"certificates";
+              //std::cerr<<"-- option 3 = "<<caCertificatesDirectory<<std::endl;
               if (!dir_test(caCertificatesDirectory)) {
-                caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"share"+G_DIR_SEPARATOR_S+"certificates";
-                //std::cerr<<"-- option 5 = "<<caCertificatesDirectory<<std::endl;
+                caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"etc"+G_DIR_SEPARATOR_S+"grid-security"+G_DIR_SEPARATOR_S+"certificates";
+                //std::cerr<<"-- option 4 = "<<caCertificatesDirectory<<std::endl;
                 if (!dir_test(caCertificatesDirectory)) {
-                  caCertificatesDirectory = std::string(G_DIR_SEPARATOR_S)+"etc"+G_DIR_SEPARATOR_S+"grid-security"+G_DIR_SEPARATOR_S+"certificates";
-                  //std::cerr<<"-- option 6 = "<<caCertificatesDirectory<<std::endl;
+                  caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"share"+G_DIR_SEPARATOR_S+"certificates";
+                  //std::cerr<<"-- option 5 = "<<caCertificatesDirectory<<std::endl;
                   if (!dir_test(caCertificatesDirectory)) {
-                    if(require) {
-                      logger.msg(WARNING,
-                        "Can not find CA certificates directory in default locations:\n"
-                        "~/.arc/certificates, ~/.globus/certificates,\n"
-                        "%s/etc/certificates, %s/etc/grid-security/certificates,\n"
-                        "%s/share/certificates, /etc/grid-security/certificates.\n"
-                        "The certificate will not be verified.\n"
-                        "If the CA certificates directory does exist, please manually specify the locations via env\n"
-                        "X509_CERT_DIR, or the cacertificatesdirectory item in client.conf\n",
-                        ArcLocation::Get(), ArcLocation::Get(), ArcLocation::Get());
-                      res = false;
+                    caCertificatesDirectory = std::string(G_DIR_SEPARATOR_S)+"etc"+G_DIR_SEPARATOR_S+"grid-security"+G_DIR_SEPARATOR_S+"certificates";
+                    //std::cerr<<"-- option 6 = "<<caCertificatesDirectory<<std::endl;
+                    if (!dir_test(caCertificatesDirectory)) {
+                      if(require) {
+                        logger.msg(WARNING,
+                          "Can not find CA certificates directory in default locations:\n"
+                          "~/.arc/certificates, ~/.globus/certificates,\n"
+                          "%s/etc/certificates, %s/etc/grid-security/certificates,\n"
+                          "%s/share/certificates, /etc/grid-security/certificates.\n"
+                          "The certificate will not be verified.\n"
+                          "If the CA certificates directory does exist, please manually specify the locations via env\n"
+                          "X509_CERT_DIR, or the cacertificatesdirectory item in client.conf\n",
+                          ArcLocation::Get(), ArcLocation::Get(), ArcLocation::Get());
+                        res = false;
+                      }
+                      caCertificatesDirectory.clear();
                     }
-                    caCertificatesDirectory.clear();
                   }
                 }
               }
@@ -826,6 +860,7 @@ namespace Arc {
           }
           HANDLESTRATT("cacertificatepath", CACertificatePath)
           HANDLESTRATT("cacertificatesdirectory", CACertificatesDirectory)
+	  HANDLESTRATT("causesystem", CAUseSystem)
           if (common["certificatelifetime"]) {
             certificateLifeTime = Period((std::string)common["certificatelifetime"]);
             common["certificatelifetime"].Destroy();
@@ -961,6 +996,7 @@ namespace Arc {
       file << "cacertificatepath = " << caCertificatePath << std::endl;
     if (!caCertificatesDirectory.empty())
       file << "cacertificatesdirectory = " << caCertificatesDirectory << std::endl;
+    file << "causesystem = " << (caUseSystem?1:0) << std::endl;
     if (certificateLifeTime > 0)
       file << "certificatelifetime = " << certificateLifeTime << std::endl;
     if (slcs)
@@ -1253,15 +1289,18 @@ static std::string cert_file_fix(const std::string& old_file,std::string& new_fi
     GET_OLD_VAR("X509_USER_CERT",x509_user_cert_old,x509_user_cert_set);
     GET_OLD_VAR("X509_USER_PROXY",x509_user_proxy_old,x509_user_proxy_set);
     GET_OLD_VAR("X509_CERT_DIR",ca_cert_dir_old,ca_cert_dir_set);
+    GET_OLD_VAR("X509_CERT_POLICY",ca_cert_policy_old,ca_cert_policy_set);
     SET_NEW_VAR_FILE("X509_USER_KEY",cfg.KeyPath(),x509_user_key_new);
     SET_NEW_VAR_FILE("X509_USER_CERT",cfg.CertificatePath(),x509_user_cert_new);
     SET_NEW_VAR_FILE("X509_USER_PROXY",cfg.ProxyPath(),x509_user_proxy_new);
     SET_NEW_VAR("X509_CERT_DIR",cfg.CACertificatesDirectory());
+    SET_NEW_VAR("X509_CERT_POLICY",(cfg.CAUseSystem()?"system":"grid"));
     EnvLockWrap(false);
   }
 
   CertEnvLocker::~CertEnvLocker(void) {
     EnvLockUnwrap(false);
+    SET_OLD_VAR("X509_CERT_POLICY",ca_cert_policy_old,ca_cert_policy_set);
     SET_OLD_VAR("X509_CERT_DIR",ca_cert_dir_old,ca_cert_dir_set);
     SET_OLD_VAR("X509_USER_PROXY",x509_user_proxy_old,x509_user_proxy_set);
     SET_OLD_VAR("X509_USER_CERT",x509_user_cert_old,x509_user_cert_set);
