@@ -135,6 +135,7 @@ namespace Arc {
    * - proxypath / ProxyPath(const std::string&)
    * - cacertificatesdirectory / CACertificatesDirectory(const std::string&)
    * - cacertificatepath / CACertificatePath(const std::string&)
+   * - causesystem / CAUseSystem(bool)
    * - timeout / Timeout(int)
    * - joblist / JobListFile(const std::string&)
    * - joblisttype / JobListType(const std::string&)
@@ -386,7 +387,8 @@ namespace Arc {
      * @see InitializeCredentials()
      **/
     bool CredentialsFound() const {
-      return !((proxyPath.empty() && (certificatePath.empty() || keyPath.empty())) || caCertificatesDirectory.empty());
+      return !((proxyPath.empty() && (certificatePath.empty() || keyPath.empty())) ||
+               (caCertificatesDirectory.empty() && !caUseSystem));
     }
 
     /// Load specified configuration file
@@ -486,7 +488,7 @@ namespace Arc {
     
     /// Set type of job storage
     /**
-     * Possible storage types are BDB and XML. This method always return true.
+     * Possible storage types are SQLITE and XML. This method always return true.
      * 
      * The attribute associated with this setter method is 'joblisttype'.
      * 
@@ -709,7 +711,7 @@ namespace Arc {
 
     /// Set authentication token.
     /**
-     * Assigns OToken (authenticatio and sometimes authorizzation token)
+     * Assigns OToken (authentication and sometimes authorization token)
      * to be used in HTTP communication.
      *
      * @param cred The credential represented as a string
@@ -921,6 +923,60 @@ namespace Arc {
      **/
     const std::string& CACertificatesDirectory() const { return caCertificatesDirectory; }
 
+    /// Set if system wide CA-certificates to be used
+    /**
+     * Specifies if system wide CA-certificates are used.
+     *
+     * The attribute associated with this setter method is
+     * 'causesystem'.
+     *
+     * @param newCAUseSystem if system wide CA-certificates are used.
+     * @return This method always returns \c true.
+     * @see InitializeCredentials()
+     * @see CredentialsFound() const
+     **/
+    bool CAUseSystem(bool newCAUseSystem) { caUseSystem = newCAUseSystem; return true; }
+    bool CAUseSystem(const std::string& newCAUseSystem) {
+      if((newCAUseSystem == "true") || (newCAUseSystem == "1"))
+        return CAUseSystem(true);
+      if((newCAUseSystem == "false") || (newCAUseSystem == "0"))
+        return CAUseSystem(false);
+      return false;
+    }
+    /**
+     * Retrieve if system wide CA-cerificates are used.
+     *
+     * @return True if system wide CA-certificate are used.
+     * @see InitializeCredentials()
+     * @see CredentialsFound() const
+     **/
+    bool CAUseSystem() const { return caUseSystem; }
+
+    bool TLSAllowInsecure(bool newTLSAllowInsecure) { tlsAllowInsecure = newTLSAllowInsecure; return true; }
+    bool TLSAllowInsecure() const { return tlsAllowInsecure; }
+
+    enum AuthType {
+      AuthTypeUndefined = 0, /// No restrictions, use whatever authentication avaialble (default)
+      AuthTypeNone = 1,      /// Do not use any credentials (anonymous)
+      AuthTypeCert = 2,      /// Use X.509 certificate/proxy for authentication
+      AuthTypeToken = 3      /// Use OIDC/OAuth2 token for authentication (bearer token)
+    };
+
+    /**
+     * Specify which authentication type to use for communication.
+     *
+     * @param newAuthType authentication type to use for communication.
+     * @return This method always returns \c true.
+     **/
+    bool CommunicationAuthType(AuthType newAuthType) { authType = newAuthType; return true; } 
+    /**
+     * Retrieve which authentication type to use for communication.
+     *
+     * @return Authentication type for communication.
+     **/
+    AuthType CommunicationAuthType() const { return authType; }
+
+
     /// Set certificate life time
     /**
      * Sets lifetime of user certificate which will be obtained from
@@ -1335,6 +1391,9 @@ namespace Arc {
     int keySize;
     std::string caCertificatePath;
     std::string caCertificatesDirectory;
+    bool caUseSystem;
+    bool tlsAllowInsecure;
+    AuthType authType;
     Period certificateLifeTime;
 
     URL slcs;
@@ -1398,6 +1457,8 @@ namespace Arc {
     bool x509_user_proxy_set;
     std::string ca_cert_dir_old;
     bool ca_cert_dir_set;
+    std::string ca_cert_policy_old;
+    bool ca_cert_policy_set;
   };
 
   /** @} */
