@@ -159,12 +159,23 @@ namespace Arc {
         if (!cfg.proxy.empty()) comp.NewChild("ProxyPath") = cfg.proxy;
         if (!cfg.credential.empty()) comp.NewChild("Credential") = cfg.credential;
       };
-      if (!cfg.cafile.empty()) comp.NewChild("CACertificatePath") = cfg.cafile;
-      if (!cfg.cadir.empty()) {
-        XMLNode cadir = comp.NewChild("CACertificatesDir");
-        cadir = cfg.cadir;
-        cadir.NewAttribute("PolicyGlobus") = "true";
+      if(!cfg.systemca) {
+        if (!cfg.cafile.empty()) comp.NewChild("CACertificatePath") = cfg.cafile;
+        if (!cfg.cadir.empty()) {
+          XMLNode cadir = comp.NewChild("CACertificatesDir");
+          cadir = cfg.cadir;
+          cadir.NewAttribute("PolicyGlobus") = "true";
+        };
+      } else {
+        comp.NewChild("SystemCA") = "true";
       };
+      if (sec.ver == UseCredVerify) {
+        comp.NewChild("AllowInsecure") = "false";
+      } else if (sec.ver == NoCredVerify) {
+        comp.NewChild("AllowInsecure") = "true";
+      } else if(cfg.tlsallowinsecure) {
+        comp.NewChild("AllowInsecure") = "true";
+      }
       comp.NewAttribute("entry") = "tls";
       if (sec.sec == SSL3Sec) comp.NewChild("Handshake") = "SSLv3";
       else if (sec.sec == TLS10Sec) comp.NewChild("Handshake") = "TLSv1.0";
@@ -348,7 +359,7 @@ namespace Arc {
     return port;
   }
 
-  static TCPSec http_url_to_sec(const URL& url, bool have_otoken) {
+  static TCPSec http_url_to_sec(const URL& url, bool use_otoken, bool use_x509) {
     TCPSec sec;
     if(url.Protocol() == "https" || url.Protocol() == "davs") {
       if(url.Option("protocol") == "ssl3") {
@@ -378,18 +389,26 @@ namespace Arc {
     } else if(url.Option("encryption") == "off") {
       sec.enc = NoEnc;
     }
-    if(have_otoken) {
+    if(use_otoken) {
       sec.cred = UseNoCred;
-    } else {
+    } else if(use_x509) {
       sec.cred = UseX509Cred;
+    } else {
+      sec.cred = UseNoCred; // no authentication is going to be used
     }
-    // URL option overrides decision taken by presence of token
+    // URL option overrides decision taken by configuration
     std::string credOption = url.Option("tlscred");
     if(credOption == "none") {
       sec.cred = UseNoCred;
     } else if(credOption == "x509") {
       sec.cred = UseX509Cred;
     }
+    std::string noVerifyOption = url.Option("allowinsecure");
+    if(noVerifyOption == "off") {
+      sec.ver = UseCredVerify;
+    } else if(noVerifyOption == "on") {
+      sec.ver = NoCredVerify;
+    }
     return sec;
   }
 
@@ -399,14 +418,14 @@ namespace Arc {
     : ClientTCP(cfg,
                 get_http_proxy_host(url,proxy_host,proxy_port),
                 get_http_proxy_port(url,proxy_host,proxy_port),
-                http_url_to_sec(url,!cfg.otoken.empty()),
+                http_url_to_sec(url,!cfg.otoken.empty() && cfg.otoken_for_auth,cfg.cert_for_auth),
                 timeout,
                 url.Option("tcpnodelay") == "yes"),
       http_entry(NULL),
       default_url(url),
       relative_uri(url.Option("relativeuri") == "yes"),
       encoded_uri(url.Option("encodeduri") != "no"),
-      sec(http_url_to_sec(url,!cfg.otoken.empty())),
+      sec(http_url_to_sec(url,!cfg.otoken.empty() && cfg.otoken_for_auth,cfg.cert_for_auth)),
       closed(false) {
     XMLNode comp = ConfigMakeComponent(xmlcfg["Chain"], "http.client", "http",
                      (SECURITY_IS_SSL(sec.sec)) ? "tls" :
@@ -414,7 +433,7 @@ namespace Arc {
     comp.NewAttribute("entry") = "http";
     comp.NewChild("Method") = "POST"; // Override using attributes if needed
     comp.NewChild("Endpoint") = url.str(true); // Override using attributes if needed
-    if (!cfg.otoken.empty()) comp.NewChild("Authorization") = "Bearer " + cfg.otoken; // TODO: protect and encode
+    if (!cfg.otoken.empty() && cfg.otoken_for_auth) comp.NewChild("Authorization") = "Bearer " + cfg.otoken; // TODO: protect and encode
     // Pass information about protocol and hostname to TLS level
     XMLNode compTLS = ConfigFindComponent(xmlcfg["Chain"], "tls.client", NULL);
     if(compTLS) {
@@ -535,22 +554,39 @@ namespace Arc {
     URL url(default_url);
     if (!path.empty()) {
       url.ChangeFullPath(path,true);
-      if(relative_uri) {
-        // Workaround for servers which can't handle full URLs in request
-        reqmsg.Attributes()->set("HTTP:HOST", url.Host() + ":" + tostring(url.Port()));
-        std::string rpath = encoded_uri ? url.FullPathURIEncoded() : url.FullPath();
-        if(rpath[0] != '/') rpath.insert(0,"/");
-        reqmsg.Attributes()->set("HTTP:ENDPOINT", rpath);
+    }
+    // Note: below the port is not added to Host header if port value is default one.
+    // This is not required by RFC. Such reduction is needed as workaround for broken
+    // HTTP servers which do not understand port in Host header. Luckily they seems
+    // to have no notion about TCP port at all so should always run on default port.
+    bool port_is_dropped = false;
+    if((url.Protocol() == "http") && (url.Port() == HTTP_DEFAULT_PORT)) {
+      reqmsg.Attributes()->set("HTTP:HOST", url.Host());
+      port_is_dropped = true;
+    } else if((url.Protocol() == "https") && (url.Port() == HTTPS_DEFAULT_PORT)) {
+      reqmsg.Attributes()->set("HTTP:HOST", url.Host());
+      port_is_dropped = true;
+    } else {
+      reqmsg.Attributes()->set("HTTP:HOST", url.Host() + ":" + tostring(url.Port()));
+    }
+    if(relative_uri) {
+      // Workaround for servers which can't handle full URLs in request.
+      // Here we supply only path as endpoint. And host is passed decicated
+      // HOST attributes which ends in HTTP Host header.
+      std::string rpath = encoded_uri ? url.FullPathURIEncoded() : url.FullPath();
+      if(rpath[0] != '/') rpath.insert(0,"/");
+      reqmsg.Attributes()->set("HTTP:ENDPOINT", rpath);
+    } else {
+      if(port_is_dropped) {
+        // It looks like there are also servers which compare Host header to URL in request
+	// just as text. So if port is removed from Host it need to be removed from request too.
+        int port = url.Port();
+        url.ChangePort(-1); // The special value -1 will cause port not to be added to textual representation
+        reqmsg.Attributes()->set("HTTP:ENDPOINT", url.plainstr(encoded_uri));
+        url.ChangePort(port);
       } else {
         reqmsg.Attributes()->set("HTTP:ENDPOINT", url.plainstr(encoded_uri));
       }
-    } else {
-      if(relative_uri) {
-        reqmsg.Attributes()->set("HTTP:HOST", default_url.Host() + ":" + tostring(default_url.Port()));
-        std::string rpath = encoded_uri ? default_url.FullPathURIEncoded() : default_url.FullPath();
-        if(rpath[0] != '/') rpath.insert(0,"/");
-        reqmsg.Attributes()->set("HTTP:ENDPOINT", rpath);
-      }
     }
     if (range_end != UINT64_MAX) {
       reqmsg.Attributes()->set("HTTP:Range", "bytes=" + tostring(range_start) +
