@@ -95,9 +95,11 @@ namespace Arc {
     Application = j.Application;
     Resources = j.Resources;
     DataStaging = j.DataStaging;
-
     OtherAttributes = j.OtherAttributes;
-
+    X509Delegation = j.X509Delegation;
+    TokenDelegation = j.TokenDelegation;
+    InstancesMin = j.InstancesMin;
+    InstancesMax = j.InstancesMax;
     sourceLanguage = j.sourceLanguage;
   }
 
