@@ -5,9 +5,6 @@
 #endif
 
 #include "JobInformationStorageXML.h"
-#ifdef DBJSTORE_ENABLED
-#include "JobInformationStorageBDB.h"
-#endif
 #ifdef SQLITEJSTORE_ENABLED
 #include "JobInformationStorageSQLite.h"
 #endif
@@ -17,9 +14,6 @@ namespace Arc {
   JobInformationStorageDescriptor JobInformationStorage::AVAILABLE_TYPES[] = {
 #ifdef SQLITEJSTORE_ENABLED
     { "SQLITE", &JobInformationStorageSQLite::Instance },
-#endif
-#ifdef DBJSTORE_ENABLED
-    { "BDB", &JobInformationStorageBDB::Instance },
 #endif
     { "XML", &JobInformationStorageXML::Instance },
     { NULL, NULL }
