@@ -198,121 +198,6 @@ namespace Arc {
      **/
     bool Resume();
 
-    /// Resubmit jobs
-    /**
-     * Jobs managed by this JobSupervisor will be resubmitted when invoking this
-     * method, that is the job description of a job will be tried obtained, and
-     * if successful a new job will be submitted.
-     *
-     * Before identifying jobs to be resubmitted, the
-     * JobControllerPlugin::UpdateJobs method is called for each loaded
-     * JobControllerPlugin in order to retrieve the most up to date job information.
-     * If an empty status-filter is specified, all jobs managed by this
-     * JobSupervisor will be considered for resubmission, except jobs in the
-     * undefined state (see JobState). If the status-filter is not empty, then
-     * only jobs with a general or specific state (see JobState) identical to
-     * any of the entries in the status-filter will be considered, except jobs
-     * in the undefined state. Jobs for which a job description cannot be
-     * obtained and successfully parsed will not be considered and an ERROR log
-     * message is reported, and the IDFromEndpoint URL is appended to the
-     * notresubmitted list. Job descriptions will be tried obtained either
-     * from Job object itself, or fetching them remotely. Furthermore if a Job
-     * object has the LocalInputFiles object set, then the checksum of each of
-     * the local input files specified in that object (key) will be calculated
-     * and verified to match the checksum LocalInputFiles object (value). If
-     * checksums are not matching the job will be filtered, and an ERROR log
-     * message is reported and the IDFromEndpoint URL is appended to the
-     * notresubmitted list. If no job have been identified for resubmission,
-     * false will be returned if ERRORs were reported, otherwise true is
-     * returned.
-     *
-     * The destination for jobs is partly determined by the destination
-     * parameter. If a value of 1 is specified a job will only be targeted to
-     * the execution service (ES) on which it reside. A value of 2 indicates
-     * that a job should not be targeted to the ES it currently reside.
-     * Specifying any other value will target any ES. The ESs which can be
-     * targeted are those specified in the UserConfig object of this class, as
-     * selected services. Before initiating any job submission, resource
-     * discovery and broker loading is carried out using the TargetGenerator and
-     * Broker classes, initialised by the UserConfig object of this class. If
-     * Broker loading fails, or no ExecutionTargets are found, an ERROR log
-     * message is reported and all IDFromEndpoint URLs for job considered for
-     * resubmission will be appended to the notresubmitted list and then false
-     * will be returned.
-     *
-     * When the above checks have been carried out successfully, then the
-     * Broker::Submit method will be invoked for each considered for
-     * resubmission. If it fails the IDFromEndpoint URL for the job is appended
-     * to the notresubmitted list, and an ERROR is reported. If submission
-     * succeeds the new job represented by a Job object will be appended to the
-     * resubmittedJobs list - it will not be added to this JobSupervisor. The
-     * method returns false if ERRORs were reported otherwise true is returned.
-     *
-     * @param destination specifies how target destination should be determined
-     *  (1 = same target, 2 = not same, any other = any target).
-     * @param services possible destinations for the resubmission
-     * @param resubmittedJobs list of Job objects which resubmitted jobs will be
-     *  appended to.
-     * @param rejectedURLs list of services which should be rejected
-     * @return false if any error is encountered, otherwise true.
-     **/
-    bool Resubmit(int destination, const std::list<Endpoint>& services, std::list<Job>& resubmittedJobs, const std::list<std::string>& rejectedURLs = std::list<std::string>());
-
-    /// Migrate jobs
-    /**
-     * Jobs managed by this JobSupervisor will be migrated when invoking this
-     * method, that is the job description of a job will be tried obtained, and
-     * if successful a job migration request will be sent, based on that job
-     * description.
-     *
-     * Before identifying jobs to be migrated, the
-     * JobControllerPlugin::UpdateJobs method is called for each loaded
-     * JobControllerPlugin in order to retrieve the most up to date job information.
-     * Only jobs for which the State member of the Job object has the value
-     * JobState::QUEUEING, will be considered for migration. Furthermore the job
-     * description must be obtained (either locally or remote) and successfully
-     * parsed in order for a job to be migrated. If the job description cannot
-     * be obtained or parsed an ERROR log message is reported, and the
-     * IDFromEndpoint URL of the Job object is appended to the notmigrated list.
-     * If no jobs have been identified for migration, false will be returned in
-     * case ERRORs were reported, otherwise true is returned.
-     *
-     * The execution services which can be targeted for migration are those
-     * specified in the UserConfig object of this class, as selected services.
-     * Before initiating any job migration request, resource discovery and
-     * broker* loading is carried out using the TargetGenerator and Broker
-     * classes, initialised by the UserConfig object of this class. If Broker
-     * loading fails, or no ExecutionTargets are found, an ERROR log message is
-     * reported and all IDFromEndpoint URLs for job considered for migration
-     * will be appended to the notmigrated list and then false will be returned.
-     *
-     * When the above checks have been carried out successfully, the following
-     * is done for each job considered for migration. The ActivityOldID member
-     * of the Identification member in the job description will be set to that
-     * of the Job object, and the IDFromEndpoint URL will be appended to
-     * ActivityOldID member of the job description. After that the Broker object
-     * will be used to find a suitable ExecutionTarget object, and if found a
-     * migrate request will tried sent using the ExecutionTarget::Migrate
-     * method, passing the UserConfig object of this class. The passed
-     * forcemigration boolean indicates whether the migration request at the
-     * service side should ignore failures in cancelling the existing queuing
-     * job. If the request succeeds, the corresponding new Job object is
-     * appended to the migratedJobs list. If no suitable ExecutionTarget objects
-     * are found an ERROR log message is reported and the IDFromEndpoint URL of
-     * the Job object is appended to the notmigrated list. When all jobs have
-     * been processed, false is returned if any ERRORs were reported, otherwise
-     * true.
-     *
-     * @param forcemigration indicates whether migration should succeed if
-     *  service fails to cancel the existing queuing job.
-     * @param services possible destinations for the migration
-     * @param migratedJobs list of Job objects which migrated jobs will be
-     *  appended to.
-     * @param rejectedURLs list of services which should be rejected
-     * @return false if any error is encountered, otherwise true.
-     **/
-    bool Migrate(bool forcemigration, const std::list<Endpoint>& services, std::list<Job>& migratedJobs, const std::list<std::string>& rejectedURLs = std::list<std::string>());
-
     /// Cancel jobs
     /**
      * This method cancels jobs managed by this JobSupervisor.
