@@ -69,6 +69,10 @@ namespace Arc {
           dst.AddOption("blocksize=1048576",false);
           dst.AddOption("checksum=no",false);
           DataHandle source(src, *usercfg);
+	  if (!source) {
+            logger.msg(ERROR, "Failed reading file %s", src.fullstr());
+            return false;
+	  }
           if ((!dest_handle) || (!*dest_handle) || (!(*dest_handle)->SetURL(dst))) {
             if(dest_handle) delete dest_handle;
             ((SubmitterPlugin*)this)->dest_handle = new DataHandle(dst, *usercfg);
@@ -111,13 +115,6 @@ namespace Arc {
     job.StdErr = jobdesc.Application.Error;
   }
 
-  bool SubmitterPlugin::Migrate(const std::string& /*jobid*/, const JobDescription& /*jobdesc*/, const ExecutionTarget& et,
-                                bool /*forcemigration*/, Job& /*job*/) {
-    logger.msg(INFO, "Trying to migrate to %s: Migration to a %s interface is not supported.", et.ComputingEndpoint->URLString, !supportedInterfaces.empty() ? supportedInterfaces.front() : "<unknown>");
-    return false;
-  };
-
-
   SubmitterPluginLoader::SubmitterPluginLoader() : Loader(BaseConfig().MakeConfig(Config()).Parent()) {}
 
   SubmitterPluginLoader::~SubmitterPluginLoader() {
