@@ -289,7 +289,8 @@ namespace Arc {
       try {
         // TODO (important) load credential in unified way or 
         // use already loaded one
-        Credential ci(source.GetUserConfig().ProxyPath(), source.GetUserConfig().ProxyPath(), source.GetUserConfig().CACertificatesDirectory(), "");
+        Credential ci(source.GetUserConfig().ProxyPath(), source.GetUserConfig().ProxyPath(),
+                      source.GetUserConfig().CACertificatesDirectory(), "", source.GetUserConfig().CAUseSystem());
         dn = ci.GetIdentityName();
         exp_time = ci.GetEndTime();
       } catch (CredentialError& e) {
