@@ -22,25 +22,8 @@
 
 #include "DelegationInterface.h"
 
-#ifndef X509_REQ_VERSION_1
-#define X509_REQ_VERSION_1 0
-#endif
-
-#ifndef X509_VERSION_3
-#define X509_VERSION_3 2
-#endif
-
 namespace Arc {
 
-#if (OPENSSL_VERSION_NUMBER < 0x10100000L)
-
-#define X509_getm_notAfter X509_get_notAfter
-#define X509_getm_notBefore X509_get_notBefore
-#define X509_set1_notAfter X509_set_notAfter
-#define X509_set1_notBefore X509_set_notBefore
-
-#endif
-
 #define DELEGATION_NAMESPACE "http://www.nordugrid.org/schemas/delegation"
 #define GDS10_NAMESPACE "http://www.gridsite.org/ns/delegation.wsdl"
 //#define GDS20_NAMESPACE "http://www.gridsite.org/namespaces/delegation-2"
@@ -490,7 +473,7 @@ bool DelegationConsumer::Request(std::string& content) {
       if(EVP_PKEY_set1_RSA(pkey, rsa)) {
         X509_REQ *req = X509_REQ_new();
         if(req) {
-          if(X509_REQ_set_version(req,X509_REQ_VERSION_1)) {
+          if(X509_REQ_set_version(req,0L)) {
             if(X509_REQ_set_pubkey(req,pkey)) {
               if(X509_REQ_sign(req,pkey,digest)) {
                 BIO *out = BIO_new(BIO_s_mem());
@@ -714,7 +697,7 @@ std::string DelegationProvider::Delegate(const std::string& request,const Delega
   if (!X509_set_serialNumber(cert,sno)) goto err;
   proxy_cn=tostring(ASN1_INTEGER_get(sno));
   ASN1_INTEGER_free(sno); sno=NULL;
-  X509_set_version(cert,X509_VERSION_3);
+  X509_set_version(cert,2L);
 
   /*
    Proxy certificates do not need KeyUsage extension. But
