@@ -29,8 +29,10 @@ MessageContext::~MessageContext(void) {
 
 void MessageContext::Add(const std::string& name,MessageContextElement* element) {
   MessageContextElement* old = operator[](name);
-  elements_[name]=element;
-  if(old) delete old;
+  if(old != element) {
+    elements_[name]=element;
+    if(old) delete old;
+  }
 }
 
 MessageContextElement* MessageContext::operator[](const std::string& id) {
