@@ -7,6 +7,7 @@
 #include <arc/StringConv.h>
 #include <arc/external/cJSON/cJSON.h>
 #include <arc/message/MCC.h>
+#include <arc/credential/Credential.h>
 #include <openssl/evp.h>
 #include <openssl/x509.h>
 #include <openssl/pem.h>
@@ -17,60 +18,36 @@
 #include "jwse_private.h"
 #include "openid_metadata.h"
 
-
 static EVP_PKEY* X509_get_privkey(X509*) {
   return NULL;
 }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-
-static int RSA_set0_key(RSA *r, BIGNUM *n, BIGNUM *e, BIGNUM *d) {
-  /* If the fields n and e in r are NULL, the corresponding input
-   * parameters MUST be non-NULL for n and e.  d may be
-   * left NULL (in case only the public key is used).
-   */
-  if ((r->n == NULL && n == NULL)
-      || (r->e == NULL && e == NULL))
-    return 0;
-
-  if (n != NULL) {
-    BN_free(r->n);
-    r->n = n;
-  }
-  if (e != NULL) {
-    BN_free(r->e);
-    r->e = e;
-  }
-  if (d != NULL) {
-    BN_free(r->d);
-    r->d = d;
-  }
-
-  return 1;
-}
-
-static RSA *EVP_PKEY_get0_RSA(EVP_PKEY *pkey) {
-  if(pkey)
-    if((pkey->type == EVP_PKEY_RSA) || (pkey->type == EVP_PKEY_RSA2))
-      return pkey->pkey.rsa;
-  return NULL;
-}
-
-void RSA_get0_key(const RSA *r, const BIGNUM **n, const BIGNUM **e, const BIGNUM **d) {
-  if(n) *n = NULL;
-  if(e) *e = NULL;
-  if(d) *d = NULL;
-  if(!r) return;
-  if(n) *n = r->n;
-  if(e) *e = r->e;
-  if(d) *d = r->d;
-}
-
-#endif
-
 
 namespace Arc {
  
+  class IssuerInfo {
+   private:
+    time_t const DefaultValidTime = 3600;
+   public:
+    IssuerInfo() {
+      validTill = time(nullptr) + DefaultValidTime;
+      isNonexpiring = false;
+    }
+
+    bool IsExpired() const {
+      if(isNonexpiring) return false;
+      return (static_cast< std::make_signed_t<time_t> >(time(nullptr) - validTill) > 0);
+    }
+
+    time_t validTill;
+    bool   isNonexpiring;
+    bool   isSafe;
+    Arc::AutoPointer<OpenIDMetadata> metadata;
+    Arc::AutoPointer<JWSEKeyHolderList> keys;
+  };
+
+  static std::map<std::string, IssuerInfo> issuersInfo;
+  static Glib::Mutex issuersInfoLock;
 
   char const * const JWSE::HeaderNameX509CertChain = "x5c";
   char const * const JWSE::HeaderNameJSONWebKey = "jwk";
@@ -84,45 +61,71 @@ namespace Arc {
     (void)BIO_free(bio);
   }
 
-  static EVP_PKEY* jwkECParse(cJSON* jwkObject) {
-    return NULL;
+  #define RETURNWARN(value, message) { \
+    logger.msg(WARNING, message); \
+    return value; \
   }
 
-  static EVP_PKEY* jwkRSAParse(cJSON* jwkObject) {
+  static EVP_PKEY* jwkECParse(cJSON* jwkObject, Logger& logger) {
+    cJSON* xObject = cJSON_GetObjectItem(jwkObject, "x");
+    cJSON* yObject = cJSON_GetObjectItem(jwkObject, "y");
+    cJSON* crvObject = cJSON_GetObjectItem(jwkObject, "crv");
+    if((xObject == NULL) || (yObject == NULL) || (crvObject == NULL)) RETURNWARN(NULL, "Missing x, y or crv in EC JWK");
+    if((xObject->type != cJSON_String) || (yObject->type != cJSON_String) || (crvObject->type != cJSON_String)) RETURNWARN(NULL, "Wrong type of x, y or crv in EC JWK");
+    std::string xStr = Base64::decodeURLSafe(xObject->valuestring);
+    std::string yStr = Base64::decodeURLSafe(yObject->valuestring);
+    int crvNid = EC_curve_nist2nid(crvObject->valuestring);
+    AutoPointer<EC_KEY> ecKey(EC_KEY_new_by_curve_name(crvNid),&EC_KEY_free);
+    if(!ecKey) RETURNWARN(NULL, "Failed to create EC_KEY for EC JWK");
+    AutoPointer<BIGNUM> x(NULL,&BN_free);
+    AutoPointer<BIGNUM> y(NULL,&BN_free);
+    if(!(x = BN_bin2bn(reinterpret_cast<unsigned char const *>(xStr.c_str()), xStr.length(), NULL))) RETURNWARN(NULL, "Failed to parse x parameter for EC JWK");
+    if(!(y = BN_bin2bn(reinterpret_cast<unsigned char const *>(yStr.c_str()), yStr.length(), NULL))) RETURNWARN(NULL, "Failed to parse y parameter for EC JWK");
+    if(!EC_KEY_set_public_key_affine_coordinates(ecKey.Ptr(), x.Ptr(), y.Ptr())) RETURNWARN(NULL, "Failed to assign coordinates for EC JWK");
+    x.Release(); // ??
+    y.Release(); // ??
+    AutoPointer<EVP_PKEY> evpKey(EVP_PKEY_new(), &EVP_PKEY_free);
+    if(!evpKey) RETURNWARN(NULL, "Failed to create EVP key for EC JWK");
+    if(EVP_PKEY_assign_EC_KEY(evpKey.Ptr(), ecKey.Ptr()) != 1) RETURNWARN(NULL, "Failed to assign EC_KEY key for EC JWK");
+    ecKey.Release();
+    return evpKey.Release();
+  }
+
+  static EVP_PKEY* jwkRSAParse(cJSON* jwkObject, Logger& logger) {
     cJSON* modulusObject = cJSON_GetObjectItem(jwkObject, "n");
     cJSON* exponentObject = cJSON_GetObjectItem(jwkObject, "e");
-    if((modulusObject == NULL) || (exponentObject == NULL)) return NULL;
-    if((modulusObject->type != cJSON_String) || (exponentObject->type != cJSON_String)) return NULL;
+    if((modulusObject == NULL) || (exponentObject == NULL)) RETURNWARN(NULL, "Missing e or n in RSA JWK");
+    if((modulusObject->type != cJSON_String) || (exponentObject->type != cJSON_String)) RETURNWARN(NULL, "Wrong type of r or n in RSA JWK");
     std::string modulus = Base64::decodeURLSafe(modulusObject->valuestring);
     std::string exponent = Base64::decodeURLSafe(exponentObject->valuestring);
     AutoPointer<RSA> rsaKey(RSA_new(),&RSA_free);
-    if(!rsaKey) return NULL;
+    if(!rsaKey) RETURNWARN(NULL, "Failed to create RSA key for RSA JWK");;
     AutoPointer<BIGNUM> n(NULL,&BN_free);
     AutoPointer<BIGNUM> e(NULL,&BN_free);
-    if(!(n = BN_bin2bn(reinterpret_cast<unsigned char const *>(modulus.c_str()), modulus.length(), NULL))) return NULL;
-    if(!(e = BN_bin2bn(reinterpret_cast<unsigned char const *>(exponent.c_str()), exponent.length(), NULL))) return NULL;
-    if(!RSA_set0_key(rsaKey.Ptr(), n.Ptr(), e.Ptr(), NULL)) return NULL;
+    if(!(n = BN_bin2bn(reinterpret_cast<unsigned char const *>(modulus.c_str()), modulus.length(), NULL))) RETURNWARN(NULL, "Failed to parse n parameter for RSA JWK");
+    if(!(e = BN_bin2bn(reinterpret_cast<unsigned char const *>(exponent.c_str()), exponent.length(), NULL))) RETURNWARN(NULL, "Failed to parse e parameter for RSA JWK");
+    if(!RSA_set0_key(rsaKey.Ptr(), n.Ptr(), e.Ptr(), NULL)) RETURNWARN(NULL, "Failed to assign modules and exponent for RSA JWK");
     n.Release();
     e.Release();
     AutoPointer<EVP_PKEY> evpKey(EVP_PKEY_new(), &EVP_PKEY_free);
-    if(!evpKey) return NULL;
-    if(EVP_PKEY_assign_RSA(evpKey.Ptr(), rsaKey.Ptr()) != 1) return NULL;
+    if(!evpKey) RETURNWARN(NULL, "Failed to create EVP key for RSA JWK");
+    if(EVP_PKEY_assign_RSA(evpKey.Ptr(), rsaKey.Ptr()) != 1) RETURNWARN(NULL, "Failed to assign RSA key for RSA JWK");
     rsaKey.Release();
     return evpKey.Release();
   }
 
-  static EVP_PKEY* jwkOctParse(cJSON* jwkObject) {
+  static EVP_PKEY* jwkOctParse(cJSON* jwkObject, Logger& logger) {
     cJSON* keyObject = cJSON_GetObjectItem(jwkObject, "k");
     cJSON* algObject = cJSON_GetObjectItem(jwkObject, "alg");
-    if((keyObject == NULL) || (algObject == NULL)) return NULL;
-    if((keyObject->type != cJSON_String) || (algObject->type != cJSON_String)) return NULL;
+    if((keyObject == NULL) || (algObject == NULL)) RETURNWARN(NULL, "Missing k or alg in oct JWK");
+    if((keyObject->type != cJSON_String) || (algObject->type != cJSON_String)) RETURNWARN(NULL, "Wrong type of k or ald in oct JWK");
     std::string key = Base64::decodeURLSafe(keyObject->valuestring);
     // It looks like RFC does not define any "alg" values with "JWK" usage.
     // TODO: finish implementing
-    return NULL;
+    RETURNWARN(NULL, "The oct JWK is not implemented yet");
   }
 
-  static bool jwkParse(cJSON* jwkObject, JWSEKeyHolder& keyHolder) {
+  static bool jwkParse(cJSON* jwkObject, JWSEKeyHolder& keyHolder, Logger& logger) {
     if(jwkObject->type != cJSON_Object) return false;
 
     cJSON* ktyObject = cJSON_GetObjectItem(jwkObject, "kty");
@@ -131,11 +134,11 @@ namespace Arc {
 
     EVP_PKEY* publicKey(NULL);
     if(strcmp(ktyObject->valuestring, "EC") == 0) {    
-      publicKey = jwkECParse(jwkObject);
+      publicKey = jwkECParse(jwkObject, logger);
     } else if(strcmp(ktyObject->valuestring, "RSA") == 0) {    
-      publicKey = jwkRSAParse(jwkObject);
+      publicKey = jwkRSAParse(jwkObject, logger);
     } else if(strcmp(ktyObject->valuestring, "oct") == 0) {    
-      publicKey = jwkOctParse(jwkObject);
+      publicKey = jwkOctParse(jwkObject, logger);
     }
     if(publicKey == NULL)
       return false;
@@ -259,9 +262,8 @@ namespace Arc {
     return x5cObject.Release();
   }
 
-  bool JWSE::ExtractPublicKey() const {
+  bool JWSE::ExtractPublicKey(UserConfig& userconfig) const {
     key_.Release();
-
     // So far we are going to support only embedded keys jwk and x5c and external kid.
     cJSON* x5cObject = cJSON_GetObjectItem(header_.Ptr(), HeaderNameX509CertChain);
     cJSON* jwkObject = cJSON_GetObjectItem(header_.Ptr(), HeaderNameJSONWebKey);
@@ -277,7 +279,7 @@ namespace Arc {
     } else if(jwkObject != NULL) {
       AutoPointer<JWSEKeyHolder> key(new JWSEKeyHolder());
       logger_.msg(DEBUG, "JWSE::ExtractPublicKey: jwk key");
-      if(jwkParse(jwkObject, *key)) {
+      if(jwkParse(jwkObject, *key, logger_)) {
         keyOrigin_ = EmbeddedKey;
         key_ = key;
         return true;
@@ -291,27 +293,81 @@ namespace Arc {
       if(!issuerObj || (issuerObj->type != cJSON_String))
         return false;
       bool keyProtocolSafe = true;
-      if(strncasecmp("https:", issuerObj->valuestring, 6) != 0) keyProtocolSafe = false;
-
-      OpenIDMetadata serviceMetadata;
-      OpenIDMetadataFetcher metadataFetcher(issuerObj->valuestring);
-      if(!metadataFetcher.Fetch(serviceMetadata))
-        return false;
-      char const * jwksUri = serviceMetadata.JWKSURI();
-      if(strncasecmp("https:", jwksUri, 6) != 0) keyProtocolSafe = false;
-
-      logger_.msg(DEBUG, "JWSE::ExtractPublicKey: fetching jwl key from %s", jwksUri);
-      JWSEKeyFetcher keyFetcher(jwksUri);
-      JWSEKeyHolderList keys;
-      if(!keyFetcher.Fetch(keys))
-        return false;
-      for(JWSEKeyHolderList::iterator keyIt = keys.begin(); keyIt != keys.end(); ++keyIt) {
-        if(strcmp(kidObject->valuestring, (*keyIt)->Id()) == 0) {
-          keyOrigin_ = keyProtocolSafe ? ExternalSafeKey : ExternalUnsafeKey;
-          key_ = *keyIt;
-          return true;
+      std::string issuerUrl(issuerObj->valuestring);
+	  // Issuer can be any application specific string. Typically that is URL.
+	  // Sometimes it is hostname (Google tokens). As we need URL anyway to fetch
+      // keys let's assume we have either HTTP(S) URL or hostname.	  
+      if(strncasecmp("https://", issuerUrl.c_str(), 8) == 0) {
+		// Use as is.
+	  } else if(strncasecmp("http://", issuerUrl.c_str(), 7) == 0) {
+		  keyProtocolSafe = false;
+	  } else {
+		issuerUrl = "https://" + issuerUrl + "/";
+	  }
+	  
+      {
+        Time now;
+        Arc::AutoLock<Glib::Mutex> lock(issuersInfoLock);
+        for(std::map<std::string, IssuerInfo>::iterator infoIt = issuersInfo.begin(); infoIt != issuersInfo.end();) {
+          std::map<std::string, IssuerInfo>::iterator nextIt = infoIt;
+          ++nextIt;
+          if(infoIt->second.IsExpired()) {
+            logger_.msg(DEBUG, "JWSE::ExtractPublicKey: deleting outdated info: %s", infoIt->first); 
+            issuersInfo.erase(infoIt);
+          }
+          infoIt = nextIt;
+        }
+        std::map<std::string, IssuerInfo>::iterator infoIt = issuersInfo.find(issuerObj->valuestring);
+        if(infoIt != issuersInfo.end()) {
+          for(JWSEKeyHolderList::iterator keyIt = infoIt->second.keys->begin(); keyIt != infoIt->second.keys->end(); ++keyIt) {
+            if(strcmp(kidObject->valuestring, (*keyIt)->Id()) == 0) {
+              keyOrigin_ = infoIt->second.isSafe ? ExternalSafeKey : ExternalUnsafeKey;
+              key_ = new JWSEKeyHolder(**keyIt);
+              return true;
+            }
+          }
         }
       }
+
+      Arc::AutoPointer<OpenIDMetadata> serviceMetadata(new OpenIDMetadata);
+      OpenIDMetadataFetcher metadataFetcher(issuerUrl.c_str(), userconfig);
+      if(!metadataFetcher.Fetch(*serviceMetadata))
+        return false;
+      if(serviceMetadata->Error())
+        return false;
+      char const * jwksUri = serviceMetadata->JWKSURI();
+      if(!jwksUri)
+        return false;
+
+      if(strncasecmp("https://", jwksUri, 8) != 0) keyProtocolSafe = false;
+
+      logger_.msg(DEBUG, "JWSE::ExtractPublicKey: fetching jws key from %s", jwksUri);
+      JWSEKeyFetcher keyFetcher(jwksUri, userconfig);
+      Arc::AutoPointer<JWSEKeyHolderList> keys(new JWSEKeyHolderList);
+      if(!keyFetcher.Fetch(*keys, logger_))
+        return false;
+      bool keyFound = false;
+      Time validTill;
+      for(JWSEKeyHolderList::iterator keyIt = keys->begin(); keyIt != keys->end(); ++keyIt) {
+        Time certValid = (*keyIt)->ValidTill();
+        if(keyIt == keys->begin())
+          validTill = certValid;
+        else if(certValid < validTill)
+          validTill = certValid;
+
+        if(!keyFound) {
+          if(strcmp(kidObject->valuestring, (*keyIt)->Id()) == 0) {
+            if(certValid >= Time()) {
+              keyOrigin_ = keyProtocolSafe ? ExternalSafeKey : ExternalUnsafeKey;
+              key_ = new JWSEKeyHolder(**keyIt);
+              keyFound = true;
+            }
+          }
+        }
+      }
+      SetIssuerInfo(&validTill, keyProtocolSafe, issuerObj->valuestring, serviceMetadata, keys);
+      if(keyFound)
+        return true;
     } else {
       logger_.msg(ERROR, "JWSE::ExtractPublicKey: no supported key");
       return true; // No key - no processing
@@ -320,6 +376,31 @@ namespace Arc {
 
     return false;
   }
+
+  void JWSE::SetIssuerInfo(Time* validTill, bool isSafe, std::string const& issuer, Arc::AutoPointer<OpenIDMetadata>& metadata, Arc::AutoPointer<JWSEKeyHolderList>& keys) {
+    Arc::AutoLock<Glib::Mutex> lock(issuersInfoLock);
+    IssuerInfo& info(issuersInfo[issuer]);
+    info.isSafe = isSafe;
+    info.metadata = metadata;
+    info.keys = keys;
+    if(validTill) {
+      if(*validTill < info.validTill)
+        info.validTill = validTill->GetTime();
+    } else {
+      info.isNonexpiring = true;
+    }
+  }
+
+  bool JWSE::SetIssuerInfo(Time* validTill, bool isSafe, std::string const& issuer, std::string const& metadataStr, std::string const& keysStr, Logger& logger) {
+    AutoPointer<OpenIDMetadata> metadata(new OpenIDMetadata);
+    if(!OpenIDMetadataFetcher::Import(metadataStr.c_str(), *metadata)) return false;
+
+    AutoPointer<JWSEKeyHolderList> keys(new JWSEKeyHolderList);
+    if(!JWSEKeyFetcher::Import(keysStr.c_str(), *keys, logger)) return false;
+
+    SetIssuerInfo(validTill, isSafe, issuer, metadata, keys);
+    return true;
+  }
   
   bool JWSE::InsertPublicKey(bool& keyAdded) const {
     keyAdded = false;
@@ -381,9 +462,28 @@ namespace Arc {
     }
   }
 
+  JWSEKeyHolder::JWSEKeyHolder(JWSEKeyHolder const & other): certificate_(NULL), certificateChain_(NULL), publicKey_(NULL), privateKey_(NULL) {
+    if(other.certificate_) {
+      certificate_ = X509_dup(other.certificate_);
+      publicKey_ = X509_get_pubkey(certificate_);
+      privateKey_ = X509_get_privkey(certificate_);
+    }
+    if(other.publicKey_ && !publicKey_) {
+      EVP_PKEY_up_ref(publicKey_ = other.publicKey_);
+    }
+    if(other.privateKey_ && !privateKey_) {
+      EVP_PKEY_up_ref(privateKey_ = other.privateKey_);
+    }
+    if(other.certificateChain_) {
+      certificateChain_ = X509_chain_up_ref(other.certificateChain_);
+    }
+  }
+
   JWSEKeyHolder::~JWSEKeyHolder() {
     if(publicKey_) EVP_PKEY_free(publicKey_);
     publicKey_ = NULL;
+    if(privateKey_) EVP_PKEY_free(privateKey_);
+    privateKey_ = NULL;
     if(certificate_) X509_free(certificate_);
     certificate_ = NULL;
     if(certificateChain_) sk_x509_deallocate(certificateChain_);
@@ -444,11 +544,40 @@ namespace Arc {
     certificateChain_ = certificateChain;
   }
 
-
-  JWSEKeyFetcher::JWSEKeyFetcher(char const * endpoint_url) : url_(endpoint_url), client_(Arc::MCCConfig(), url_) {
+  Time JWSEKeyHolder::ValidTill() const {
+    if(certificate_) {
+      Time certValid;
+      Period period;
+      Credential::GetLifetime(certificateChain_, certificate_, certValid, period);
+      certValid = certValid + period;
+      return certValid;
+    }
+    if(publicKey_) {
+      Time keyValid = Time() + Period(DefaultValidTime);
+      return keyValid;
+    }
+    return Time();
   }
 
-  bool JWSEKeyFetcher::Fetch(JWSEKeyHolderList& keys) {
+
+  static URL url_no_cred(char const * url_str) {
+    URL url(url_str);
+    url.AddOption("tlscred=none");
+    return url;
+  }
+
+  static Arc::MCCConfig make_config(UserConfig& userconfig) {
+    Arc::MCCConfig config;
+    userconfig.ApplyToConfig(config);
+    return config;
+  }
+
+  JWSEKeyFetcher::JWSEKeyFetcher(char const * endpoint_url, UserConfig& userconfig):
+       url_(endpoint_url?url_no_cred(endpoint_url):URL()), client_(make_config(userconfig), url_) {
+    client_.RelativeURI(true);
+  }
+
+  bool JWSEKeyFetcher::Fetch(JWSEKeyHolderList& keys, Logger& logger) {
     HTTPClientInfo info;
     PayloadRaw request;
     PayloadRawInterface* response(NULL);
@@ -475,7 +604,7 @@ namespace Arc {
         id.assign(kidObj->valuestring);
       Arc::AutoPointer<JWSEKeyHolder> key(new JWSEKeyHolder());
       if(!key) continue;
-      if(!jwkParse(keyObj, *key.Ptr()))
+      if(!jwkParse(keyObj, *key.Ptr(), logger))
         continue;
       key->Id(id.c_str());
       keys.add(key);
@@ -483,5 +612,30 @@ namespace Arc {
     return true;
   }
 
+  bool JWSEKeyFetcher::Import(char const * contentStr, JWSEKeyHolderList& keys, Logger& logger) {
+    if(!contentStr) return false;
+    AutoPointer<cJSON> content(cJSON_Parse(contentStr), &cJSON_Delete);
+    if(!content)
+      return false;
+    cJSON* keysObj = cJSON_GetObjectItem(content.Ptr(), "keys");
+    if(!keysObj || (keysObj->type != cJSON_Array))
+      return false;
+    for(int idx = 0; idx < cJSON_GetArraySize(keysObj); ++idx) {
+      cJSON* keyObj = cJSON_GetArrayItem(keysObj, idx);
+      if(!keyObj || (keyObj->type != cJSON_Object))
+        continue;
+      cJSON* kidObj = cJSON_GetObjectItem(keyObj, "kid");
+      std::string id;
+      if(kidObj && (kidObj->type == cJSON_String))
+        id.assign(kidObj->valuestring);
+      Arc::AutoPointer<JWSEKeyHolder> key(new JWSEKeyHolder());
+      if(!key) continue;
+      if(!jwkParse(keyObj, *key.Ptr(), logger))
+        continue;
+      key->Id(id.c_str());
+      keys.add(key);
+    };
+    return true;
+  }
 
 } // namespace Arc
