@@ -2,6 +2,7 @@
 #include <arc/Utils.h>
 #include <arc/URL.h>
 #include <arc/Logger.h>
+#include <arc/UserConfig.h>
 #include <arc/communication/ClientInterface.h>
 
 
@@ -13,6 +14,7 @@ namespace Arc {
   //! from OpenID Metadata.
   class OpenIDMetadata {
    public:
+    static char const * const ErrorTag;
     static char const * const IssuerTag;
     static char const * const AuthorizationEndpointTag;
     static char const * const TokenEndpointTag;
@@ -75,6 +77,8 @@ namespace Arc {
     //! Serializes stored SciToken into string container.
     bool Output(std::string& metadta) const;
 
+    char const* Error() const;
+
     char const* Issuer() const;
 
     char const* AuthorizationEndpoint() const;
@@ -155,12 +159,17 @@ namespace Arc {
 
     void Cleanup();
 
+    OpenIDMetadata(OpenIDMetadata const &);
+
+    OpenIDMetadata& operator=(OpenIDMetadata const &);
+
   }; // class OpenIDMetadata
 
   class OpenIDMetadataFetcher {
    public:
-    OpenIDMetadataFetcher(char const * issuer_url);
+    OpenIDMetadataFetcher(char const * issuer_url, UserConfig& userconfig);
     bool Fetch(OpenIDMetadata& metadata);
+    static bool Import(char const * content, OpenIDMetadata& metadata);
    private:
     URL url_;
     ClientHTTP client_;
@@ -170,12 +179,13 @@ namespace Arc {
   class OpenIDTokenFetcher {
    public:
     typedef std::list< std::pair<std::string, std::string> > TokenList;
-    OpenIDTokenFetcher(char const * token_endpoint, char const * id, char const * secret);
+    OpenIDTokenFetcher(char const * token_endpoint, UserConfig& userconfig, char const * id, char const * secret);
     bool Fetch(std::string const & grant,
                std::string const & subject,
                std::list<std::string> const & scope,
                std::list<std::string> const & audience,
                TokenList & tokens);
+    static bool Import(char const * content, TokenList & tokens);
    private:
     URL url_;
     std::string client_id_;
