@@ -154,13 +154,17 @@ bool SAMLToken::Authenticate(void) {
   return true;
 }
 
-bool SAMLToken::Authenticate(const std::string& cafile, const std::string& capath) {
+bool SAMLToken::Authenticate(const std::string& cafile, const std::string& capath, bool systemca) {
   xmlSecKeysMngr* keys_manager = NULL;
   xmlSecDSigCtx *dsigCtx;
 
   /*****************************************/
   //Verify the signature under saml:assertion
-  if((bool)x509data && (!cafile.empty() || !capath.empty())) {
+  if((bool)x509data && systemca) {
+    keys_manager = load_trusted_certs(&keys_manager, NULL, NULL);
+    if(keys_manager == NULL) { std::cerr<<"Can not load default certificates"<<std::endl; return false; } 
+  }
+  else if((bool)x509data && (!cafile.empty() || !capath.empty())) {
     keys_manager = load_trusted_certs(&keys_manager, cafile.c_str(), capath.c_str());
     //keys_manager = load_trusted_cert_file(&keys_manager, cafile.c_str());
     if(keys_manager == NULL) { std::cerr<<"Can not load trusted certificates"<<std::endl; return false; } 
@@ -244,7 +248,7 @@ SAMLToken::SAMLToken(SOAPEnvelope& soap, const std::string& certfile, const std:
       std::string current_time = t.str(Arc::UTCTime);
       assertion.NewAttribute("IssueInstant") = current_time;
 
-      Arc::Credential cred(certfile, keyfile, "", "");
+      Arc::Credential cred(certfile, keyfile, "", "", false); // it doesn't matter what kind of CA is set beause we use no CAs here
       std::string dn = cred.GetDN();
       std::string rdn = Arc::convert_to_rdn(dn);
       assertion.NewAttribute("Issuer") = rdn;
