@@ -52,6 +52,7 @@ class HTTPSecAttr: public SecAttr {
   virtual operator bool(void) const;
   virtual bool Export(SecAttrFormat format,XMLNode &val) const;
   virtual std::string get(const std::string& id) const;
+  virtual std::map< std::string, std::list<std::string> > getAll() const;
  protected:
   std::string action_;
   std::string object_;
@@ -86,6 +87,20 @@ std::string HTTPSecAttr::get(const std::string& id) const {
   return "";
 }
 
+std::map<std::string, std::list<std::string> > HTTPSecAttr::getAll() const {
+  static char const * const allIds[] = {
+    "ACTION",
+    "OBJECT",
+    NULL
+  };
+  std::map<std::string, std::list<std::string> > all;
+  for(char const * const * id = allIds; *id; ++id) {
+    std::string idStr(*id);
+    all[idStr] = SecAttr::getAll(idStr);
+  }
+  return all;
+}
+
 bool HTTPSecAttr::equal(const SecAttr &b) const {
   try {
     const HTTPSecAttr& a = (const HTTPSecAttr&)b;
@@ -176,8 +191,8 @@ static MCC_Status make_http_fault(Logger& logger,
   bool keep_alive = (!inpayload)?false:inpayload.KeepAlive();
   outpayload.KeepAlive(keep_alive);
   // Add forced headers
-  for(std::list< std::pair<std::string,std::string> >::const_iterator header = headers.begin();
-      header != headers.end();
+  for(std::list< std::pair<std::string,std::string> >::const_iterator header = headers.cbegin();
+      header != headers.cend();
       ++header) {
     outpayload.Attribute(header->first, header->second);
   }
@@ -284,19 +299,32 @@ MCC_Status MCC_HTTP_Service::process(Message& inmsg,Message& outmsg) {
   {
     std::string::size_type p = endpoint.find("://");
     if(p == std::string::npos) {
-      // TODO: Use Host attribute of HTTP
+      std::string endpoint_host = nextpayload.Attribute("host");
+      std::string endpoint_port;
+      // parse info from URI
       std::string oendpoint = nextinmsg.Attributes()->get("ENDPOINT");
       p=oendpoint.find("://");
       if(p != std::string::npos) {
         oendpoint.erase(0,p+3);
       };
-      // Assuming we have host:port here
-      if(oendpoint.empty() ||
-         (oendpoint[oendpoint.length()-1] != '/')) {
-        if(endpoint[0] != '/') oendpoint+="/";
+      // trim slash
+      oendpoint = Arc::trim(oendpoint, "/");
+      // split host:port
+      p=oendpoint.find(":");
+      std::string ohost = oendpoint;
+      if(p != std::string::npos) {
+        ohost = oendpoint.substr(0,p);
+        endpoint_port = oendpoint.substr(p);
       };
-      // TODO: HTTPS detection
-      endpoint="http://"+oendpoint+endpoint;
+      if(endpoint_host.empty()) endpoint_host = ohost;
+      // add trailing slash
+      std::string sep;
+      if (endpoint[0] != '/') sep = "/";
+      // HTTPS detection
+      std::string protocol = "http";
+      if (!inmsg.Attributes()->get("TLS:LOCALDN").empty()) protocol+="s";
+      // Construct HTTP URL endpoint
+      endpoint=protocol+"://"+endpoint_host+endpoint_port+sep+endpoint;
     };
   };
   nextinmsg.Attributes()->set("ENDPOINT",endpoint);
