@@ -16,7 +16,7 @@ using namespace Arc;
 
 BIO* BIO_new_GSIMCC(MCCInterface* mcc);
 BIO* BIO_new_GSIMCC(PayloadStreamInterface* stream);
-std::string BIO_GSIMCC_failure(BIO* bio);
+bool BIO_GSIMCC_failure(BIO* bio, MCC_Status& s);
 
 } // namespace Arc
 
