@@ -15,21 +15,6 @@
 
 namespace ArcMCCTLS {
 
-#if (OPENSSL_VERSION_NUMBER < 0x10100000L)
-
-#define X509_getm_notAfter X509_get_notAfter
-#define X509_getm_notBefore X509_get_notBefore
-#define X509_set1_notAfter X509_set_notAfter
-#define X509_set1_notBefore X509_set_notBefore
-
-#endif
-
-#if (OPENSSL_VERSION_NUMBER < 0x10002000L)
-static X509_VERIFY_PARAM *SSL_CTX_get0_param(SSL_CTX *ctx) {
-    return ctx->param;
-}
-#endif
-
 static const char * ex_data_id = "ARC_MCC_Payload_TLS";
 int PayloadTLSMCC::ex_data_index_ = -1;
 
@@ -77,7 +62,13 @@ static int verify_callback(int ok,X509_STORE_CTX *sctx) {
       default: {
         //std::cerr<<"+++ verify_callback: error: "<<X509_verify_cert_error_string(err)<<std::endl;
         if(it) {
-          it->SetFailure((std::string)X509_verify_cert_error_string(err));
+          if(it->Config().AllowInsecureConnection()) {
+            ok=1;
+            X509_STORE_CTX_set_error(sctx,X509_V_OK);
+            Logger::getRootLogger().msg(WARNING,"Ignoring verification error due to insecure connection allowed: %s",X509_verify_cert_error_string(err));
+          } else {
+            it->SetFailure((std::string)X509_verify_cert_error_string(err));
+          }
         } else {
           Logger::getRootLogger().msg(ERROR,"%s",X509_verify_cert_error_string(err));
         }
@@ -96,6 +87,8 @@ static int verify_callback(int ok,X509_STORE_CTX *sctx) {
       //std::cerr<<"+++ additional verification: subject "<<subject_name<<std::endl;
       if(it == NULL) {
         Logger::getRootLogger().msg(WARNING,"Failed to retrieve link to TLS stream. Additional policy matching is skipped.");
+      } else if(it->Config().AllowInsecureConnection()) {
+        Logger::getRootLogger().msg(WARNING,"Skipping additional policy matching due to insecure connections allowed.");
       } else {
         // Globus signing policy
         //std::cerr<<"+++ additional verification: - "<<it->Config().GlobusPolicy()<<" - "<<it->Config().CADir()<<std::endl;
@@ -452,6 +445,13 @@ PayloadTLSMCC::~PayloadTLSMCC(void) {
           // unfortunately it seems to be lost already
           // inside SSL_shutdown().
           ConfigTLSMCC::HandleError();
+        } else if(err == SSL_ERROR_SSL) {
+          // Although not explicitely specified this seems to
+          // indicate that there was already error on this SSL object
+          // and hence shutdown is meaningless.
+          // Note: it is strange that extracted error says ssl_ is still in init state
+          // despite handshake already finished. Looks like bug in OpenSSL.
+          ConfigTLSMCC::HandleError();
         } else {
           // This case is unexpected. So it is better to
           // report it.
