@@ -146,8 +146,11 @@ ArcSec::SecHandlerStatus LegacySecHandler::Handle(Arc::Message* msg) const {
     for(std::list<std::string>::const_iterator grp = groups.begin(); grp != groups.end(); ++grp) {
       const char* vo = auth.get_group_vo(*grp);
       const voms_t* voms = auth.get_group_voms(*grp);
+      const otokens_t* otokens = auth.get_group_otokens(*grp);
+      //std::string glid = auth.get_group_globalid(*grp);
       std::list<std::string> vos;
       std::list<std::string> vomss;
+      std::list<std::string> otokenss;
       if((vo != NULL) && (*vo != '\0')) vos.push_back(vo);
       if(voms != NULL) {
         for(std::vector<voms_fqan_t>::const_iterator f = voms->fqans.begin();
@@ -157,7 +160,14 @@ ArcSec::SecHandlerStatus LegacySecHandler::Handle(Arc::Message* msg) const {
           vomss.push_back(fqan);
         };
       };
-      sattr->AddGroup(*grp, vos, vomss);
+      // We need something like fqan for tokens. Currently we only need to identify cleint.
+      // For that combination of subject and issuer is enough.
+      if(otokens) {
+        if(!otokens->subject.empty() && !otokens->issuer.empty()) {
+          otokenss.push_back(otokens->issuer + "/" + otokens->subject);
+        };
+      };
+      sattr->AddGroup(*grp, vos, vomss, otokenss);
     };
   };
 
