@@ -83,6 +83,7 @@ AuthUser::source_t AuthUser::sources[] = {
   { "file", &AuthUser::match_file },
   { "voms", &AuthUser::match_voms },
   { "authtokens", &AuthUser::match_otokens },
+  { "authtokensgen", &AuthUser::match_ftokens },
   { "userlist", &AuthUser::match_vo },
   { "plugin", &AuthUser::match_plugin },
   { NULL, NULL }
@@ -134,9 +135,10 @@ AuthUser::AuthUser(Arc::Message& message):
     otokens_t otokens;
     otokens.subject  = sattr->get("sub");
     otokens.issuer   = sattr->get("iss");
-    otokens.audience = sattr->get("aud");
+    otokens.audiences= sattr->getAll("aud");
     Arc::tokenize(sattr->get("scope"), otokens.scopes);
     otokens.groups   = sattr->getAll("wlcg.groups");
+    otokens.claims   = sattr->getAll();
     otokens_data_.push_back(otokens);
     if(subject_.empty())
       subject_ = sattr->get("iss+sub");
@@ -146,9 +148,10 @@ AuthUser::AuthUser(Arc::Message& message):
     otokens_t otokens;
     otokens.subject  = sattr->get("sub");
     otokens.issuer   = sattr->get("iss");
-    otokens.audience = sattr->get("aud");
+    otokens.audiences= sattr->getAll("aud");
     Arc::tokenize(sattr->get("scope"), otokens.scopes);
     otokens.groups   = sattr->getAll("wlcg.groups");
+    otokens.claims   = sattr->getAll();
     otokens_data_.push_back(otokens);
     if(subject_.empty())
       subject_ = sattr->get("iss+sub");
