@@ -50,6 +50,7 @@ class SAMLAssertionSecAttr: public Arc::SecAttr {
   virtual bool Export(SecAttrFormat format,XMLNode &val) const;
   virtual bool Import(SecAttrFormat format, const XMLNode& val);
   virtual std::string get(const std::string& id) const;
+  virtual std::map< std::string,std::list<std::string> > getAll() const;
  protected:
   virtual bool equal(const SecAttr &b) const;
  private:
@@ -71,6 +72,10 @@ std::string SAMLAssertionSecAttr::get(const std::string& id) const {
   return "";
 }
 
+std::map<std::string, std::list<std::string> > SAMLAssertionSecAttr::getAll() const {
+ return std::map<std::string, std::list<std::string> >();
+}
+
 bool SAMLAssertionSecAttr::equal(const SecAttr& b) const {
   try {
     const SAMLAssertionSecAttr& a = dynamic_cast<const SAMLAssertionSecAttr&>(b);
@@ -142,7 +147,7 @@ bool SAMLAssertionSecAttr::Import(Arc::SecAttrFormat format, const XMLNode& val)
   return false;
 }
 
-SAMLTokenSH::SAMLTokenSH(Config *cfg,ChainContext*,Arc::PluginArgument* parg):SecHandler(cfg,parg),valid_(false){
+SAMLTokenSH::SAMLTokenSH(Config *cfg,ChainContext*,Arc::PluginArgument* parg):SecHandler(cfg,parg),system_ca_(false),valid_(false){
   if(!init_xmlsec()) return;
   process_type_=process_none;
   std::string process_type = (std::string)((*cfg)["Process"]);
@@ -159,7 +164,8 @@ SAMLTokenSH::SAMLTokenSH(Config *cfg,ChainContext*,Arc::PluginArgument* parg):Se
     };
     ca_file_=(std::string)((*cfg)["CACertificatePath"]);
     ca_dir_=(std::string)((*cfg)["CACertificatesDir"]);
-    if(ca_file_.empty() && ca_dir_.empty()) {
+    system_ca_ = (((std::string)((*cfg)["SystemCA"])) == "true");
+    if(ca_file_.empty() && ca_dir_.empty() && !system_ca_) {
       logger.msg(WARNING,"Both of CACertificatePath and CACertificatesDir elements missing or empty");
     };
     aa_service_ = (std::string)((*cfg)["AAService"]);
@@ -171,7 +177,8 @@ SAMLTokenSH::SAMLTokenSH(Config *cfg,ChainContext*,Arc::PluginArgument* parg):Se
     //trusted ca.
     ca_file_=(std::string)((*cfg)["CACertificatePath"]);
     ca_dir_=(std::string)((*cfg)["CACertificatesDir"]);
-    if(ca_file_.empty() && ca_dir_.empty()) {
+    system_ca_ = (((std::string)((*cfg)["SystemCA"])) == "true");
+    if(ca_file_.empty() && ca_dir_.empty() && !system_ca_) {
       logger.msg(INFO,"Missing or empty CertificatePath or CACertificatesDir element; will only check the signature, will not do message authentication");
     };
     process_type_=process_extract;
@@ -180,7 +187,7 @@ SAMLTokenSH::SAMLTokenSH(Config *cfg,ChainContext*,Arc::PluginArgument* parg):Se
     return;
   };
   if(!cert_file_.empty()) {
-    Arc::Credential cred(cert_file_, key_file_, ca_dir_, ca_file_);
+    Arc::Credential cred(cert_file_, key_file_, ca_dir_, ca_file_, system_ca_);
     local_dn_ = convert_to_rdn(cred.GetDN());
   }
   valid_ = true;
@@ -205,7 +212,7 @@ SecHandlerStatus SAMLTokenSH::Handle(Arc::Message* msg) const {
         return false;
       };
 */
-      if((!ca_file_.empty() || !ca_dir_.empty()) && !st.Authenticate(ca_file_, ca_dir_)) {
+      if((!ca_file_.empty() || !ca_dir_.empty()) && !st.Authenticate(ca_file_, ca_dir_, system_ca_)) {
         logger.msg(ERROR, "Failed to authenticate SAML Token inside the incoming SOAP");
         return false;
       };
@@ -267,6 +274,7 @@ SecHandlerStatus SAMLTokenSH::Handle(Arc::Message* msg) const {
         if (!key_file_.empty()) cfg.AddPrivateKey(key_file_);
         if (!ca_file_.empty()) cfg.AddCAFile(ca_file_);
         if (!ca_dir_.empty()) cfg.AddCADir(ca_dir_);
+	cfg.SetSystemCA(system_ca_);
 
         Arc::ClientSOAP client(cfg, aa_service_url);
         Arc::PayloadSOAP *response = NULL;
@@ -296,7 +304,7 @@ SecHandlerStatus SAMLTokenSH::Handle(Arc::Message* msg) const {
 
         std::string resp_idname = "ID";
         Arc::XMLSecNode attr_resp_secnode(attr_resp);
-        if(attr_resp_secnode.VerifyNode(resp_idname, ca_file_, ca_dir_)) {
+        if(attr_resp_secnode.VerifyNode(resp_idname, ca_file_, ca_dir_, system_ca_, true)) {
           logger.msg(Arc::INFO, "Succeeded to verify the signature under <samlp:Response/>");
         }
         else {
@@ -317,7 +325,7 @@ SecHandlerStatus SAMLTokenSH::Handle(Arc::Message* msg) const {
         Arc::XMLNode assertion = attr_resp["saml:Assertion"];
         std::string assertion_idname = "ID";
         Arc::XMLSecNode assertion_secnode(assertion);
-        if(assertion_secnode.VerifyNode(assertion_idname, ca_file_, ca_dir_)) {
+        if(assertion_secnode.VerifyNode(assertion_idname, ca_file_, ca_dir_, system_ca_, true)) {
           logger.msg(Arc::INFO, "Succeeded to verify the signature under <saml:Assertion/>");
         }
         else {
