@@ -31,7 +31,7 @@ class UsernameTokenSH : public SecHandler {
   UsernameTokenSH(Arc::Config *cfg, Arc::ChainContext* ctx, Arc::PluginArgument* parg);
   virtual ~UsernameTokenSH(void);
   static Arc::Plugin* get_sechandler(Arc::PluginArgument* arg);
-  virtual bool Handle(Arc::Message* msg) const;
+  virtual SecHandlerStatus Handle(Arc::Message* msg) const;
   operator bool(void) { return valid_; };
   bool operator!(void) { return !valid_; };
 };
