@@ -79,6 +79,9 @@ namespace DataStaging {
         durl = dtr->get_cache_file();
         caching = true;
       }
+      else if (!dtr->get_mapped_destination().empty()) {
+        durl = dtr->get_mapped_destination();
+      }
       int child_uid = 0;
       int child_gid = 0;
       if(!caching) {
@@ -92,41 +95,52 @@ namespace DataStaging {
       // Check if credentials are needed for source/dest
       Arc::DataHandle surl_h(surl, dtr->get_usercfg());
       Arc::DataHandle durl_h(durl, dtr->get_usercfg());
-      if (!dtr->get_usercfg().CredentialString().empty() &&
-          surl_h && !surl_h->RequiresCredentialsInFile() &&
-          durl_h && !durl_h->RequiresCredentialsInFile()) {
+
+      bool needCredentialsInFile = (surl_h && surl_h->RequiresCredentialsInFile()) ||
+                                   (durl_h && durl_h->RequiresCredentialsInFile());
+      if (!needCredentialsInFile) {
         // If file-based credentials are not required then send through stdin
-        stdin_ = dtr->get_usercfg().CredentialString();
+        if (!dtr->get_usercfg().OToken().empty()) {
+          stdin_ = "token ";
+	  stdin_ += dtr->get_usercfg().OToken();
+	} else {
+          stdin_ = "x509 ";
+          stdin_ += dtr->get_usercfg().CredentialString();
+        }
       } else {
         // If child is going to be run under different user ID
         // we must ensure it will be able to read credentials.
         tmp_proxy_ = prepare_proxy(dtr->get_usercfg().ProxyPath(), child_uid, child_gid);
         if (!tmp_proxy_.empty()) {
           args.push_back("--sopt");
-          args.push_back("credential="+tmp_proxy_);
+          args.push_back(std::string("credential=")+tmp_proxy_);
           args.push_back("--dopt");
-          args.push_back("credential="+tmp_proxy_);
+          args.push_back(std::string("credential=")+tmp_proxy_);
         } else if(!dtr->get_usercfg().ProxyPath().empty()) {
           args.push_back("--sopt");
-          args.push_back("credential="+dtr->get_usercfg().ProxyPath());
+          args.push_back(std::string("credential=")+dtr->get_usercfg().ProxyPath());
           args.push_back("--dopt");
-          args.push_back("credential="+dtr->get_usercfg().ProxyPath());
+          args.push_back(std::string("credential=")+dtr->get_usercfg().ProxyPath());
         }
       }
       if (!dtr->get_usercfg().CACertificatesDirectory().empty()) {
         args.push_back("--sopt");
-        args.push_back("ca="+dtr->get_usercfg().CACertificatesDirectory());
+        args.push_back(std::string("ca=")+dtr->get_usercfg().CACertificatesDirectory());
         args.push_back("--dopt");
-        args.push_back("ca="+dtr->get_usercfg().CACertificatesDirectory());
+        args.push_back(std::string("ca=")+dtr->get_usercfg().CACertificatesDirectory());
       }
+      args.push_back("--sopt");
+      args.push_back(std::string("casystem=")+Arc::tostring((int)dtr->get_usercfg().CAUseSystem()));
+      args.push_back("--dopt");
+      args.push_back(std::string("casystem=")+Arc::tostring((int)dtr->get_usercfg().CAUseSystem()));
       args.push_back("--topt");
-      args.push_back("minspeed="+Arc::tostring(transfer_params.min_current_bandwidth));
+      args.push_back(std::string("minspeed=")+Arc::tostring(transfer_params.min_current_bandwidth));
       args.push_back("--topt");
-      args.push_back("minspeedtime="+Arc::tostring(transfer_params.averaging_time));
+      args.push_back(std::string("minspeedtime=")+Arc::tostring(transfer_params.averaging_time));
       args.push_back("--topt");
-      args.push_back("minavgspeed="+Arc::tostring(transfer_params.min_average_bandwidth));
+      args.push_back(std::string("minavgspeed=")+Arc::tostring(transfer_params.min_average_bandwidth));
       args.push_back("--topt");
-      args.push_back("maxinacttime="+Arc::tostring(transfer_params.max_inactivity_time));
+      args.push_back(std::string("maxinacttime=")+Arc::tostring(transfer_params.max_inactivity_time));
 
       if (dtr->get_source()->CheckSize()) {
         args.push_back("--size");
@@ -179,7 +193,7 @@ namespace DataStaging {
         return;
       }
     }
-    handler_->Add(this);
+    GetHandler().Add(this);
   }
 
   DataDeliveryLocalComm::~DataDeliveryLocalComm(void) {
@@ -191,7 +205,11 @@ namespace DataStaging {
       }
     }
     if(!tmp_proxy_.empty()) Arc::FileDelete(tmp_proxy_);
-    if(handler_) handler_->Remove(this);
+    GetHandler().Remove(this);
+  }
+
+  std::string DataDeliveryLocalComm::DeliveryId() const {
+    return "localhost";
   }
 
   void DataDeliveryLocalComm::PullStatus(void) {
