@@ -7,6 +7,8 @@
 
 #include "DataDeliveryRemoteComm.h"
 
+#define DELEGATION_NAMESPACE "http://www.nordugrid.org/schemas/delegation"
+
 namespace DataStaging {
 
   Arc::Logger DataDeliveryRemoteComm::logger(Arc::Logger::getRootLogger(), "DataStaging.DataDeliveryRemoteComm");
@@ -53,6 +55,9 @@ namespace DataStaging {
       durl = dtr->get_cache_file();
       caching = true;
     }
+    else if (!dtr->get_mapped_destination().empty()) {
+      durl = dtr->get_mapped_destination();
+    }
 
     if (dtr->host_cert_for_remote_delivery()) {
       Arc::initializeCredentialsType cred_type(Arc::initializeCredentialsType::TryCredentials);
@@ -65,6 +70,8 @@ namespace DataStaging {
 
     // connect to service and make a new transfer request
     logger_->msg(Arc::VERBOSE, "Connecting to Delivery service at %s", endpoint.str());
+    // TODO: implement pool of ClientSOAP objects instead of having one for each Comm
+    // object. That shall reduce number of TCP connections. 
     client = new Arc::ClientSOAP(cfg, endpoint, timeout);
 
     Arc::NS ns;
@@ -149,17 +156,21 @@ namespace DataStaging {
 
     delete response;
     valid = true;
-    handler_->Add(this);
+    GetHandler().Add(this);
   }
 
   DataDeliveryRemoteComm::~DataDeliveryRemoteComm() {
     // If transfer is still going, send cancellation request to service
     if (valid) CancelDTR();
-    if (handler_) handler_->Remove(this);
+    GetHandler().Remove(this);
     Glib::Mutex::Lock lock(lock_);
     delete client;
   }
 
+  std::string DataDeliveryRemoteComm::DeliveryId() const {
+    return endpoint.str();
+  }
+
   void DataDeliveryRemoteComm::CancelDTR() {
     Glib::Mutex::Lock lock(lock_);
     if (!client) return;
@@ -481,8 +492,9 @@ namespace DataStaging {
     const std::string& cert = (!usercfg.ProxyPath().empty() ? usercfg.ProxyPath() : usercfg.CertificatePath());
     const std::string& key  = (!usercfg.ProxyPath().empty() ? usercfg.ProxyPath() : usercfg.KeyPath());
     const std::string& credentials = usercfg.CredentialString();
+    const std::string& token = usercfg.OToken();
 
-    if (credentials.empty() && (key.empty() || cert.empty())) {
+    if (credentials.empty() && (key.empty() || cert.empty()) && token.empty()) {
       logger_->msg(Arc::VERBOSE, "Failed locating credentials");
       return false;
     }
@@ -498,18 +510,27 @@ namespace DataStaging {
       return false;
     }
 
-    Arc::DelegationProviderSOAP * deleg = NULL;
-    // Use in-memory credentials if set in UserConfig
-    if (!credentials.empty()) deleg = new Arc::DelegationProviderSOAP(credentials);
-    else deleg = new Arc::DelegationProviderSOAP(cert, key);
-    logger_->msg(Arc::VERBOSE, "Initiating delegation procedure");
-    if (!deleg->DelegateCredentialsInit(*entry, &(client->GetContext()))) {
-      logger_->msg(Arc::VERBOSE, "Failed to initiate delegation credentials");
+    if(token.empty()) {
+      Arc::DelegationProviderSOAP * deleg = NULL;
+      // Use in-memory credentials if set in UserConfig
+      if (!credentials.empty()) deleg = new Arc::DelegationProviderSOAP(credentials);
+      else deleg = new Arc::DelegationProviderSOAP(cert, key);
+      logger_->msg(Arc::VERBOSE, "Initiating delegation procedure");
+      if (!deleg->DelegateCredentialsInit(*entry, &(client->GetContext()))) {
+        logger_->msg(Arc::VERBOSE, "Failed to initiate delegation credentials");
+        delete deleg;
+        return false;
+      }
+      deleg->DelegatedToken(op);
       delete deleg;
-      return false;
+    } else {
+      Arc::NS ns; ns["deleg"]=DELEGATION_NAMESPACE;
+      Arc::XMLNode deleg_token = op.NewChild("deleg:DelegatedToken",ns);
+      deleg_token.NewAttribute("deleg:Format")="token";
+      deleg_token.NewChild("deleg:Id")="";
+      deleg_token.NewChild("deleg:Value")=token;
     }
-    deleg->DelegatedToken(op);
-    delete deleg;
+
     return true;
   }
 
