@@ -139,6 +139,8 @@ int main(int argc,char* argv[]) {
   std::string dest_cred_path;
   std::string source_ca_path;
   std::string dest_ca_path;
+  bool source_ca_system = false;
+  bool dest_ca_system = false;
   OptionParser opt;
   opt.AddOption(0,"surl","","source URL",source_str);
   opt.AddOption(0,"durl","","destination URL",dest_str);
@@ -176,6 +178,8 @@ int main(int argc,char* argv[]) {
         source_cred_path = o->substr(p+1);
       } else if(name == "ca") {
         source_ca_path = o->substr(p+1);
+      } else if(name == "casystem") {
+	source_ca_system = (o->substr(p+1) == "1");
       } else {
         source_url.AddOption(*o);
       };
@@ -192,6 +196,8 @@ int main(int argc,char* argv[]) {
         dest_cred_path = o->substr(p+1);
       } else if(name == "ca") {
         dest_ca_path = o->substr(p+1);
+      } else if(name == "casystem") {
+	dest_ca_system = (o->substr(p+1) == "1");
       } else {
         dest_url.AddOption(*o);
       };
@@ -236,12 +242,23 @@ int main(int argc,char* argv[]) {
   // Read credential from stdin if available
   std::string proxy_cred;
   std::getline(std::cin, proxy_cred, '\0');
+  bool is_x509_cred = false;
+  bool is_token_cred = false;
+  if(strncmp(proxy_cred.c_str(), "x509 ", 5) == 0) {
+     is_x509_cred = true;
+     proxy_cred.erase(0,5);
+  } else if(strncmp(proxy_cred.c_str(), "token ", 6) == 0) {
+     is_token_cred = true;
+     proxy_cred.erase(0,6);
+  }
 
   initializeCredentialsType source_cred(initializeCredentialsType::SkipCredentials);
   UserConfig source_cfg(source_cred);
-  if(!source_cred_path.empty()) source_cfg.ProxyPath(source_cred_path);
-  else if (!proxy_cred.empty()) source_cfg.CredentialString(proxy_cred);
+  if(!source_cred_path.empty()) { source_cfg.ProxyPath(source_cred_path); }
+  else if(is_x509_cred) { source_cfg.CredentialString(proxy_cred); }
+  else if(is_token_cred) { source_cfg.OToken(proxy_cred); }
   if(!source_ca_path.empty()) source_cfg.CACertificatesDirectory(source_ca_path);
+  source_cfg.CAUseSystem(source_ca_system);
   //source_cfg.UtilsDirPath(...); - probably not needed
   DataHandle source(source_url, source_cfg);
   if(!source) {
@@ -259,8 +276,10 @@ int main(int argc,char* argv[]) {
   initializeCredentialsType dest_cred(initializeCredentialsType::SkipCredentials);
   UserConfig dest_cfg(dest_cred);
   if(!dest_cred_path.empty()) dest_cfg.ProxyPath(dest_cred_path);
-  else if (!proxy_cred.empty()) dest_cfg.CredentialString(proxy_cred);
+  else if(is_x509_cred) dest_cfg.CredentialString(proxy_cred);
+  else if(is_token_cred) dest_cfg.OToken(proxy_cred);
   if(!dest_ca_path.empty()) dest_cfg.CACertificatesDirectory(dest_ca_path);
+  dest_cfg.CAUseSystem(dest_ca_system);
   //dest_cfg.UtilsDirPath(...); - probably not needed
   DataHandle dest(dest_url,dest_cfg);
   if(!dest) {
@@ -279,6 +298,7 @@ int main(int argc,char* argv[]) {
   if (!source_cfg.ProxyPath().empty()) {
     SetEnv("X509_USER_PROXY", source_cfg.ProxyPath());
     if (!source_cfg.CACertificatesDirectory().empty()) SetEnv("X509_CERT_DIR", source_cfg.CACertificatesDirectory());
+    if (source_cfg.CAUseSystem()) SetEnv("X509_CERT_POLICY", "system");
     // those tools also use hostcert by default if the user is root...
     if (getuid() == 0) {
       SetEnv("X509_USER_CERT", source_cfg.ProxyPath());
