@@ -22,6 +22,7 @@
 #include <arc/FileUtils.h>
 #include <arc/Utils.h>
 #include <arc/otokens/openid_metadata.h>
+#include <arc/otokens/otokens.h>
 #include "grid-manager/log/JobLog.h"
 #include "grid-manager/log/JobsMetrics.h"
 #include "grid-manager/log/HeartBeatMetrics.h"
@@ -43,24 +44,6 @@ static const std::string BES_ARC_NAMESPACE("http://www.nordugrid.org/schemas/a-r
 static const std::string DELEG_ARC_NPREFIX("arcdeleg");
 static const std::string DELEG_ARC_NAMESPACE("http://www.nordugrid.org/schemas/delegation");
 
-static const std::string ES_TYPES_NPREFIX("estypes");
-static const std::string ES_TYPES_NAMESPACE("http://www.eu-emi.eu/es/2010/12/types");
-
-static const std::string ES_CREATE_NPREFIX("escreate");
-static const std::string ES_CREATE_NAMESPACE("http://www.eu-emi.eu/es/2010/12/creation/types");
-
-static const std::string ES_DELEG_NPREFIX("esdeleg");
-static const std::string ES_DELEG_NAMESPACE("http://www.eu-emi.eu/es/2010/12/delegation/types");
-
-static const std::string ES_RINFO_NPREFIX("esrinfo");
-static const std::string ES_RINFO_NAMESPACE("http://www.eu-emi.eu/es/2010/12/resourceinfo/types");
-
-static const std::string ES_MANAG_NPREFIX("esmanag");
-static const std::string ES_MANAG_NAMESPACE("http://www.eu-emi.eu/es/2010/12/activitymanagement/types");
-
-static const std::string ES_AINFO_NPREFIX("esainfo");
-static const std::string ES_AINFO_NAMESPACE("http://www.eu-emi.eu/es/2010/12/activity/types");
-
 char const* ARexService::InfoPath = "*info";
 char const* ARexService::LogsPath = "*logs";
 char const* ARexService::NewPath = "*new";
@@ -90,6 +73,7 @@ class ARexSecAttr: public Arc::SecAttr {
   virtual operator bool(void) const;
   virtual bool Export(Arc::SecAttrFormat format,Arc::XMLNode &val) const;
   virtual std::string get(const std::string& id) const;
+  virtual std::map< std::string, std::list<std::string> > getAll() const;
   void SetResource(const std::string& service, const std::string& job, const std::string& action);
  protected:
   std::string action_;
@@ -119,70 +103,6 @@ ARexSecAttr::ARexSecAttr(const Arc::XMLNode op) {
       id_=JOB_POLICY_OPERATION_URN;
       action_=JOB_POLICY_OPERATION_MODIFY;
     }
-  } else if(MatchXMLNamespace(op,ES_CREATE_NAMESPACE)) {
-    if(MatchXMLName(op,"CreateActivity")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_CREATE;
-    }
-  } else if(MatchXMLNamespace(op,ES_DELEG_NAMESPACE)) {
-    if(MatchXMLName(op,"InitDelegation")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_CREATE;
-    } else if(MatchXMLName(op,"PutDelegation")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_MODIFY;
-    } else if(MatchXMLName(op,"GetDelegationInfo")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_READ;
-    }
-  } else if(MatchXMLNamespace(op,ES_RINFO_NAMESPACE)) {
-    if(MatchXMLName(op,"GetResourceInfo")) {
-      id_=AREX_POLICY_OPERATION_URN;
-      action_=AREX_POLICY_OPERATION_INFO;
-    } else if(MatchXMLName(op,"QueryResourceInfo")) {
-      id_=AREX_POLICY_OPERATION_URN;
-      action_=AREX_POLICY_OPERATION_INFO;
-    }
-  } else if(MatchXMLNamespace(op,ES_MANAG_NAMESPACE)) {
-    if(MatchXMLName(op,"PauseActivity")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_MODIFY;
-    } else if(MatchXMLName(op,"ResumeActivity")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_MODIFY;
-    } else if(MatchXMLName(op,"ResumeActivity")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_MODIFY;
-    } else if(MatchXMLName(op,"NotifyService")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_MODIFY;
-    } else if(MatchXMLName(op,"CancelActivity")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_MODIFY;
-    } else if(MatchXMLName(op,"WipeActivity")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_MODIFY;
-    } else if(MatchXMLName(op,"RestartActivity")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_MODIFY;
-    } else if(MatchXMLName(op,"GetActivityStatus")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_READ;
-    } else if(MatchXMLName(op,"GetActivityInfo")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_READ;
-    }
-  } else if(MatchXMLNamespace(op,ES_AINFO_NAMESPACE)) {
-    if(MatchXMLName(op,"ListActivities")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_READ;
-    } else if(MatchXMLName(op,"GetActivityStatus")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_READ;
-    } else if(MatchXMLName(op,"GetActivityInfo")) {
-      id_=JOB_POLICY_OPERATION_URN;
-      action_=JOB_POLICY_OPERATION_READ;
-    }
   }
 }
 
@@ -236,6 +156,16 @@ std::string ARexSecAttr::get(const std::string& id) const {
   return "";
 };
 
+std::map< std::string, std::list<std::string> > ARexSecAttr::getAll() const {
+  std::map< std::string, std::list<std::string> > all;
+  all["ACTION"] = Arc::SecAttr::getAll("ACTION");
+  all["NAMESPACE"] = Arc::SecAttr::getAll("NAMESPACE");
+  all["SERVICE"] = Arc::SecAttr::getAll("SERVICE");
+  all["JOB"] = Arc::SecAttr::getAll("JOB");
+  all["FILE"] = Arc::SecAttr::getAll("FILE");
+  return all;
+}
+
 static bool match_lists(const std::list<std::pair<bool,std::string> >& list1, const std::list<std::string>& list2, std::string& matched) {
   for(std::list<std::pair<bool,std::string> >::const_iterator l1 = list1.begin(); l1 != list1.end(); ++l1) {
     for(std::list<std::string>::const_iterator l2 = list2.begin(); l2 != list2.end(); ++l2) {
@@ -273,33 +203,6 @@ static bool match_groups(std::list<std::pair<bool,std::string> > const & groups,
   return false;
 }
 
-static Arc::XMLNode ESCreateResponse(Arc::PayloadSOAP& res,const char* opname) {
-  Arc::XMLNode response = res.NewChild(ES_CREATE_NPREFIX + ":" + opname + "Response");
-  return response;
-}
-
-/*
-static Arc::XMLNode ESDelegResponse(Arc::PayloadSOAP& res,const char* opname) {
-  Arc::XMLNode response = res.NewChild(ES_DELEG_NPREFIX + ":" + opname + "Response");
-  return response;
-}
-*/
-
-static Arc::XMLNode ESRInfoResponse(Arc::PayloadSOAP& res,const char* opname) {
-  Arc::XMLNode response = res.NewChild(ES_RINFO_NPREFIX + ":" + opname + "Response");
-  return response;
-}
-
-static Arc::XMLNode ESManagResponse(Arc::PayloadSOAP& res,const char* opname) {
-  Arc::XMLNode response = res.NewChild(ES_MANAG_NPREFIX + ":" + opname + "Response");
-  return response;
-}
-
-static Arc::XMLNode ESAInfoResponse(Arc::PayloadSOAP& res,const char* opname) {
-  Arc::XMLNode response = res.NewChild(ES_AINFO_NPREFIX + ":" + opname + "Response");
-  return response;
-}
-
 //static Arc::LogStream logcerr(std::cerr);
 
 static Arc::Plugin* get_service(Arc::PluginArgument* arg) {
@@ -342,18 +245,6 @@ CountedResource::CountedResource(int maxconsumers):
 CountedResource::~CountedResource(void) {
 }
 
-class CountedResourceLock {
- private:
-  CountedResource& r_;
- public:
-  CountedResourceLock(CountedResource& resource):r_(resource) {
-    r_.Acquire();
-  };
-  ~CountedResourceLock(void) {
-    r_.Release();
-  };
-};
-
 static std::string GetPath(std::string url){
   std::string::size_type ds, ps;
   ds=url.find("//");
@@ -450,7 +341,6 @@ static std::string GetPath(Arc::Message &inmsg,std::string &base) {
 
 #define SOAP_NOT_SUPPORTED { \
   logger_.msg(Arc::ERROR, "SOAP operation is not supported: %s", op.Name()); \
-  delete outpayload; \
   return make_soap_fault(outmsg,"Operation not supported"); \
 }
 
@@ -476,18 +366,17 @@ Arc::MCC_Status ARexService::preProcessSecurity(Arc::Message& inmsg,Arc::Message
     };
   };
 
-  // Process grid-manager configuration if not done yet
-  config = ARexConfigContext::GetRutimeConfiguration(inmsg, config_, uname_, endpoint_);
+  // Check main authorization rules
+  std::list<std::pair<bool,std::string> > const & groups = config_.MatchingGroups();
+  if(!groups.empty()) {
+    if(match_groups(groups, inmsg)) {
+      // Process grid-manager configuration if not done yet
+      config = ARexConfigContext::GetRutimeConfiguration(inmsg, config_, uname_, endpoint_);
+    };
+  };
   if(!config) {
     // Service is not operational except public information.
     // But public information also has own authorization rules
-    if(!config_.PublicInformationEnabled()) {
-      logger_.msg(Arc::VERBOSE, "Can't obtain configuration. Public information is disabled.");
-      char const* fault = "User can't be assigned configuration";
-      return is_soap ?
-        make_soap_fault(outmsg, fault) :
-        make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault);
-    };
     // Check additional authorization rules
     std::list<std::pair<bool,std::string> > const & groups = config_.MatchingGroupsPublicInformation();
     if(!groups.empty()) {
@@ -643,86 +532,16 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
     if(id.empty()) {
       // Factory operations
       logger_.msg(Arc::VERBOSE, "process: factory endpoint");
-      Arc::PayloadSOAP* outpayload = new Arc::PayloadSOAP(ns_);
-      Arc::PayloadSOAP& res = *outpayload;
-      // Preparing known namespaces
-      outpayload->Namespaces(ns_);
-      if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_CREATE_NAMESPACE)) {
-        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
-        // Applying known namespaces
-        inpayload->Namespaces(ns_);
-        if(MatchXMLName(op,"CreateActivity")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESCreateActivities(*config,op,ESCreateResponse(res,"CreateActivity"),clientid);
-        } else {
-          SOAP_NOT_SUPPORTED;
-        }
-      } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_RINFO_NAMESPACE)) {
-        // Aplying known namespaces
-        // Resource information is available for anonymous requests - null config is handled properly
-        inpayload->Namespaces(ns_);
-        if(MatchXMLName(op,"GetResourceInfo")) {
-          CountedResourceLock cl_lock(infolimit_);
-          ESGetResourceInfo(*config,op,ESRInfoResponse(res,"GetResourceInfo"));
-        } else if(MatchXMLName(op,"QueryResourceInfo")) {
-          CountedResourceLock cl_lock(infolimit_);
-          ESQueryResourceInfo(*config,op,ESRInfoResponse(res,"QueryResourceInfo"));
-        } else {
-          SOAP_NOT_SUPPORTED;
-        }
-      } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_MANAG_NAMESPACE)) {
-        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
-        // Aplying known namespaces
-        inpayload->Namespaces(ns_);
-        if(MatchXMLName(op,"PauseActivity")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESPauseActivity(*config,op,ESManagResponse(res,"PauseActivity"));
-        } else if(MatchXMLName(op,"ResumeActivity")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESResumeActivity(*config,op,ESManagResponse(res,"ResumeActivity"));
-        } else if(MatchXMLName(op,"NotifyService")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESNotifyService(*config,op,ESManagResponse(res,"NotifyService"));
-        } else if(MatchXMLName(op,"CancelActivity")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESCancelActivity(*config,op,ESManagResponse(res,"CancelActivity"));
-        } else if(MatchXMLName(op,"WipeActivity")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESWipeActivity(*config,op,ESManagResponse(res,"WipeActivity"));
-        } else if(MatchXMLName(op,"RestartActivity")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESRestartActivity(*config,op,ESManagResponse(res,"RestartActivity"));
-        } else if(MatchXMLName(op,"GetActivityStatus")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESGetActivityStatus(*config,op,ESManagResponse(res,"GetActivityStatus"));
-        } else if(MatchXMLName(op,"GetActivityInfo")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESGetActivityInfo(*config,op,ESManagResponse(res,"GetActivityInfo"));
-        } else {
-          SOAP_NOT_SUPPORTED;
-        }
-      } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_AINFO_NAMESPACE)) {
-        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
-        // Aplying known namespaces
-        inpayload->Namespaces(ns_);
-        if(MatchXMLName(op,"ListActivities")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESListActivities(*config,op,ESAInfoResponse(res,"ListActivities"));
-        } else if(MatchXMLName(op,"GetActivityStatus")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESGetActivityStatus(*config,op,ESAInfoResponse(res,"GetActivityStatus"));
-        } else if(MatchXMLName(op,"GetActivityInfo")) {
-          CountedResourceLock cl_lock(beslimit_);
-          ESGetActivityInfo(*config,op,ESAInfoResponse(res,"GetActivityInfo"));
-        } else {
-          SOAP_NOT_SUPPORTED;
-        }
-      } else if(config_.ARCInterfaceEnabled() && MatchXMLNamespace(op,BES_ARC_NAMESPACE)) {
+      if(config_.ARCInterfaceEnabled() && MatchXMLNamespace(op,BES_ARC_NAMESPACE)) {
         if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"CacheCheck")) {
+          Arc::PayloadSOAP* outpayload = new Arc::PayloadSOAP(ns_);
+          // Preparing known namespaces
+          outpayload->Namespaces(ns_);
           CacheCheck(*config,*inpayload,*outpayload);
+          outmsg.Payload(outpayload);
         } else {
           SOAP_NOT_SUPPORTED;
         }
@@ -730,6 +549,9 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
+        Arc::PayloadSOAP* outpayload = new Arc::PayloadSOAP(ns_);
+        // Preparing known namespaces
+        outpayload->Namespaces(ns_);
         CountedResourceLock cl_lock(beslimit_);
         std::string credentials;
         if(!delegation_stores_.Process(config->GmConfig().DelegationDir(),
@@ -743,51 +565,18 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
             // ARC delegation is done per job but stored under
             // own id. So storing must be done outside processing code.
             UpdateCredentials(*config,op,outpayload->Child(),credentials);
-          } else if(MatchXMLNamespace(op,ES_DELEG_NAMESPACE)) {
-            // ES has delegations assigned their own ids and are
-            // already updated in delegation_stores_.Process()
-#if 1
-            // But for compatibility during intermediate period store delegations in
-            // per-job proxy file too.
-            if(op.Name() == "PutDelegation") {
-              // PutDelegation
-              //   DelegationID
-              //   Credential
-              std::string id = op["DelegationId"];
-              if(!id.empty()) {
-                DelegationStore& delegation_store(delegation_stores_[config->GmConfig().DelegationDir()]);
-                std::list<std::string> job_ids;
-                if(delegation_store.GetLocks(id,config->GridName(),job_ids)) {
-                  for(std::list<std::string>::iterator job_id = job_ids.begin(); job_id != job_ids.end(); ++job_id) {
-                    // check if that is main delegation for this job
-                    std::string delegationid;
-                    if(job_local_read_delegationid(*job_id,config->GmConfig(),delegationid)) {
-                      if(id == delegationid) {
-                        std::string credentials;
-                        if(delegation_store.GetCred(id,config->GridName(),credentials)) {
-                          if(!credentials.empty()) {
-                            GMJob job(*job_id,Arc::User(config->User().get_uid()));
-                            (void)job_proxy_write_file(job,config->GmConfig(),credentials);
-                          };
-                        };
-                      };
-                    };
-                  }; 
-                };
-              };
-            };
-#endif
           };
         };
+        outmsg.Payload(outpayload);
       } else {
         SOAP_NOT_SUPPORTED;
       };
       if(logger_.getThreshold() <= Arc::VERBOSE) {
         std::string str;
-        outpayload->GetDoc(str, true);
+        Arc::PayloadSOAP* oupayload = dynamic_cast<Arc::PayloadSOAP*>(outmsg.Payload());
+        if(oupayload) oupayload->GetDoc(str, true);
         logger_.msg(Arc::VERBOSE, "process: response=%s",str);
       };
-      outmsg.Payload(outpayload);
     } else {
       // Listing operations for session directories
       // TODO: proper failure like interface is not supported
@@ -1061,12 +850,6 @@ class ArexServiceNamespaces: public Arc::NS {
     Arc::NS& ns_(*this);
     ns_[BES_ARC_NPREFIX]=BES_ARC_NAMESPACE;
     ns_[DELEG_ARC_NPREFIX]=DELEG_ARC_NAMESPACE;
-    ns_[ES_TYPES_NPREFIX]=ES_TYPES_NAMESPACE;
-    ns_[ES_CREATE_NPREFIX]=ES_CREATE_NAMESPACE;
-    ns_[ES_DELEG_NPREFIX]=ES_DELEG_NAMESPACE;
-    ns_[ES_RINFO_NPREFIX]=ES_RINFO_NAMESPACE;
-    ns_[ES_MANAG_NPREFIX]=ES_MANAG_NAMESPACE;
-    ns_[ES_AINFO_NPREFIX]=ES_AINFO_NAMESPACE;
     ns_["wsa"]="http://www.w3.org/2005/08/addressing";
     ns_["jsdl"]="http://schemas.ggf.org/jsdl/2005/11/jsdl";
     ns_["wsrf-bf"]="http://docs.oasis-open.org/wsrf/bf-2";
@@ -1196,6 +979,43 @@ ARexService::ARexService(Arc::Config *cfg,Arc::PluginArgument *parg):Arc::Servic
       logger.getRootLogger().addDestinations(dests);
     }
   }
+
+  {
+    std::string base_path = config_.ControlDir()+"/tokenissuers";
+    DIR* dir = ::opendir(base_path.c_str());
+    if(dir) {
+      for(;;) {
+        struct dirent* d = ::readdir(dir);
+        if(!d) break;
+        if(strcmp(d->d_name,".") == 0) continue;
+        if(strcmp(d->d_name,"..") == 0) continue;
+        std::string issuer_path   = base_path + "/" + d->d_name + "/issuer";
+        std::string metadata_path = base_path + "/" + d->d_name + "/metadata";
+        std::string keys_path     = base_path + "/" + d->d_name + "/keys";
+	std::string issuer;
+	std::string metadata;
+	std::string keys;
+        if(Arc::FileRead(issuer_path, issuer) &&
+           Arc::FileRead(metadata_path, metadata) &&
+           Arc::FileRead(keys_path, keys)) {
+          if(!issuer.empty() && !metadata.empty() && !keys.empty()) {
+            // Assign validity enough to the future
+            if(Arc::JWSE::SetIssuerInfo(nullptr, true, issuer, metadata, keys, logger_)) {
+              logger_.msg(Arc::INFO, "Created entry for JWT issuer %s", d->d_name);
+	    } else {
+              logger_.msg(Arc::ERROR, "Failed to create entry for JWT issuer %s", d->d_name);
+            }
+          } else {
+            logger_.msg(Arc::ERROR, "Empty data for JWT issuer %s", d->d_name);
+          }
+        } else {
+          logger_.msg(Arc::ERROR, "Failed to read data for JWT issuer %s", d->d_name);
+        }
+      }
+      closedir(dir);
+    }
+  }
+
   valid=true;
 }
 
