@@ -2,6 +2,7 @@
 #define __ARC_AREX_H__
 
 #include <arc/message/PayloadRaw.h>
+#include <arc/message/PayloadSOAP.h>
 #include <arc/delegation/DelegationInterface.h>
 #include <arc/infosys/InformationInterface.h>
 #include <arc/Thread.h>
@@ -35,6 +36,18 @@ class CountedResource {
   void Release(void);
 };
 
+class CountedResourceLock {
+ private:
+  CountedResource& r_;
+ public:
+  CountedResourceLock(CountedResource& resource):r_(resource) {
+    r_.Acquire();
+  };
+  ~CountedResourceLock(void) {
+    r_.Release();
+  };
+};
+
 class OptimizedInformationContainer: public Arc::InformationContainer {
  private:
   bool parse_xml_;
@@ -86,20 +99,6 @@ class ARexService: public Arc::Service {
   /** Update credentials for specified job through A-REX own interface */
   Arc::MCC_Status UpdateCredentials(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out,const std::string& credentials);
 
-  // EMI ES operations
-  Arc::MCC_Status ESCreateActivities(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out,const std::string& clientid);
-  AREXOP(ESGetResourceInfo);
-  AREXOP(ESQueryResourceInfo);
-  AREXOP(ESPauseActivity);
-  AREXOP(ESResumeActivity);
-  AREXOP(ESNotifyService);
-  AREXOP(ESCancelActivity);
-  AREXOP(ESWipeActivity);
-  AREXOP(ESRestartActivity);
-  AREXOP(ESListActivities);
-  AREXOP(ESGetActivityStatus);
-  AREXOP(ESGetActivityInfo);
-
   // HTTP operations
   Arc::MCC_Status GetJob(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath);
   Arc::MCC_Status GetLogs(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath);
@@ -202,8 +201,5 @@ class ARexService: public Arc::Service {
 
 } // namespace ARex
 
-#define HTTP_ERR_NOT_SUPPORTED (501)
-#define HTTP_ERR_FORBIDDEN (403)
-
 #endif
 
