@@ -13,9 +13,6 @@
 #include <arc/FileUtils.h>
 
 #include "FileRecordSQLite.h"
-#ifdef HAVE_DBCXX
-#include "FileRecordBDB.h"
-#endif
 
 #include "DelegationStore.h"
 
@@ -29,11 +26,6 @@ namespace ARex {
     mtimeout_ = 0;
     mrec_ = NULL;
     switch(db) {
-#ifdef HAVE_DBCXX
-      case DbBerkeley:
-        fstore_ = new FileRecordBDB(base, allow_recover);
-        break;
-#endif
       case DbSQLite:
         fstore_ = new FileRecordSQLite(base, allow_recover);
         break;
@@ -68,11 +60,6 @@ namespace ARex {
             };
           };
           switch(db) {
-#ifdef HAVE_DBCXX
-            case DbBerkeley:
-              fstore_ = new FileRecordBDB(base);
-              break;
-#endif
             case DbSQLite:
               fstore_ = new FileRecordSQLite(base);
               break;
@@ -342,6 +329,10 @@ namespace ARex {
     return fstore_->Find(id,client,meta);
   }
 
+  std::string DelegationStore::FindCred(const std::string& id,const std::string& client,std::list<std::string>& meta) {
+    return fstore_->Find(id,client,meta);
+  }
+
   bool DelegationStore::GetCred(const std::string& id, const std::string& client, std::string& credentials) {
     std::list<std::string> meta;
     std::string path = fstore_->Find(id,client,meta);
@@ -349,7 +340,6 @@ namespace ARex {
       failure_ = "Local error - failed to find specified credentials. "+fstore_->Error();
       return false;
     }
-    std::string content;
     if(!Arc::FileRead(path,credentials)) {
       failure_ = "Local error - failed to read credentials";
       return false;
@@ -375,6 +365,19 @@ namespace ARex {
     return res;
   }
 
+  std::list<std::pair<std::string, std::list<std::string> > > DelegationStore::ListCredInfos(const std::string& client) {
+    std::list<std::pair<std::string, std::list<std::string> > > res;
+    FileRecord::Iterator& rec = *(fstore_->NewIterator());
+    for(;(bool)rec;++rec) {
+      if(rec.owner() != client) continue;
+      res.push_back(std::pair<std::string, std::list<std::string> >());
+      res.back().first =  rec.id();
+      res.back().second =  rec.meta();
+    };
+    delete &rec;
+    return res;
+  }
+
   std::list<std::pair<std::string,std::string> > DelegationStore::ListLockedCredIDs(const std::string& lock_id) {
     std::list<std::pair<std::string,std::string> > ids;
     (void)fstore_->ListLocked(lock_id, ids);
@@ -470,5 +473,34 @@ namespace ARex {
     return true;
   }
 
+  bool DelegationStore::PutCred(std::string& id,const std::string& client,const std::string& credentials,const std::list<std::string>& meta) {
+    if(!id.empty()) {
+      std::list<std::string> old_meta;
+      std::string path = fstore_->Find(id,client,old_meta);
+      if(path.empty()) {
+        failure_ = "Local error - failed to find delegation slot. "+fstore_->Error();
+        return false;
+      }
+      if(!Arc::FileCreate(path,credentials,0,0,S_IRUSR|S_IWUSR)) {
+        failure_ = "Local error - failed to store credentials";
+        return false;
+      };
+      if(!meta.empty())
+        fstore_->Modify(id,client,meta);
+    } else {
+      std::string path = fstore_->Add(id,client,meta);
+      if(path.empty()) {
+        failure_ = "Local error - failed to create slot for delegation. "+fstore_->Error();
+        return false;
+      }
+      if(!Arc::FileCreate(path,credentials,0,0,S_IRUSR|S_IWUSR)) {
+        fstore_->Remove(id,client);
+        failure_ = "Local error - failed to store credentials";
+        return false;
+      };
+    };
+    return true;
+  }
+
 } // namespace ARex
 
