@@ -5,20 +5,16 @@
 
 namespace ARex {
 
+class JobsList;
 class DTRGenerator;
-class CommFIFO;
 class GMConfig;
-class sleep_st;
 
 class GridManager {
  private:
   Arc::SimpleCounter active_;
   bool tostop_;
-  Arc::SimpleCondition* sleep_cond_;
-  CommFIFO* wakeup_interface_;
   GMConfig& config_;
-  sleep_st* wakeup_;
-  DTRGenerator* dtr_generator_;
+  JobsList* jobs_;
   GridManager();
   GridManager(const GridManager&);
   static void grid_manager(void* arg);
@@ -27,6 +23,7 @@ class GridManager {
   GridManager(GMConfig& config);
   ~GridManager(void);
   operator bool(void) { return (active_.get()>0); };
+  void RequestJobAttention(const std::string& job_id);
 };
 
 } // namespace ARex
