@@ -99,7 +99,7 @@ bool CoreConfig::ParseConfINI(GMConfig& config, Arc::ConfigFile& cfile) {
   static const int emies_secnum       = 3;
   cf.AddSection("arex/ws/jobs");
   static const int publicinfo_secnum  = 4;
-  cf.AddSection("arex/ws/publicinfo");
+  cf.AddSection("infosys/accesscontrol");
   static const int ws_secnum          = 5;
   cf.AddSection("arex/ws");
   static const int jura_secnum        = 6;
@@ -364,6 +364,9 @@ bool CoreConfig::ParseConfINI(GMConfig& config, Arc::ConfigFile& cfile) {
             logger.msg(Arc::ERROR, "Wrong option in delegationdb"); return false;
           };
         }
+        else if (command == "usetokenforvoms") {
+          if (!CheckYesNoCommand(config.wlcg_to_voms, command, rest)) return false;
+        }
         else if (command == "forcedefaultvoms") {
           std::string str = rest;
           if (str.empty()) {
@@ -371,6 +374,42 @@ bool CoreConfig::ParseConfINI(GMConfig& config, Arc::ConfigFile& cfile) {
           }
           config.forced_voms[""] = str;
         }
+        else if (command == "tokenscopes") {
+          std::string str = rest;
+          std::list<std::string> pairs;
+          Arc::tokenize(str,pairs,",");
+          for(std::list<std::string>::iterator pair = pairs.begin(); pair != pairs.end(); ++pair) {
+            std::string::size_type pos = pair->find('=');
+            if(pos != std::string::npos) {
+              // action=scope
+              config.token_scopes[Arc::trim(pair->substr(0,pos))].push_back(Arc::trim(pair->substr(pos+1)));
+            } else {
+              // shortcut
+              std::string shortcut = Arc::trim(*pair);
+	      if(shortcut == "wlcg") {
+                config.token_scopes["jobinfo"].push_back("compute.read");
+                config.token_scopes["jobcreate"].push_back("compute.create");
+                config.token_scopes["jobcancel"].push_back("compute.cancel");
+                config.token_scopes["jobdelete"].push_back("compute.cancel");
+                config.token_scopes["datainfo"].push_back("compute.read");
+                config.token_scopes["datawrite"].push_back("compute.modify");
+                config.token_scopes["dataread"].push_back("compute.read");
+              }
+            }
+          }
+        }
+        else if (command == "authtokenmap") {
+          if (config.job_log) {
+            std::list<std::string> pairs;
+            Arc::tokenize(rest, pairs, ",");
+            for(std::list<std::string>::iterator pair = pairs.begin(); pair != pairs.end(); ++pair) {
+              std::string::size_type seppos = pair->find(':');
+              if(seppos != std::string::npos) {
+                config.job_log->AddTokenMap(pair->substr(0,seppos),pair->substr(seppos+1));
+              }
+            }
+          }
+        }
         /*
           #infoproviders_timelimit
           Currently information provider timeout is not implemented,
@@ -465,7 +504,6 @@ bool CoreConfig::ParseConfINI(GMConfig& config, Arc::ConfigFile& cfile) {
 
     if (cf.SectionNum() == publicinfo_secnum) { // arex/ws/publicinfo
       if (cf.SubSection()[0] == '\0') {
-        config.enable_publicinfo = true;
         if (command == "allowaccess") {
           while(!rest.empty()) {
             std::string str = Arc::ConfigIni::NextArg(rest);
@@ -526,6 +564,18 @@ bool CoreConfig::ParseConfINI(GMConfig& config, Arc::ConfigFile& cfile) {
             config.job_log->SetCredentials("", "", jobreport_cadir);
           }
         }
+        else if (command == "vomsless_vo") {
+          if (config.job_log) {
+            std::string vomsless_vo = rest;
+            std::string authgroup;
+            std::string::size_type seppos = vomsless_vo.find(' ');
+            if(seppos != std::string::npos) {
+              authgroup = vomsless_vo.substr(0, seppos);
+              vomsless_vo.erase(0, seppos+1);
+            }
+            config.job_log->AddVomslessVo(authgroup, vomsless_vo);
+          }
+        }
       };
       continue;
     };
