@@ -99,12 +99,12 @@ void GMConfig::SetDefaults() {
 
   enable_arc_interface = false;
   enable_emies_interface = false;
-  enable_publicinfo = false;
 
   cert_dir = Arc::GetEnv("X509_CERT_DIR");
   voms_dir = Arc::GetEnv("X509_VOMS_DIR");
 
   sshfs_mounts_enabled = false;
+  wlcg_to_voms = false;
 }
 
 bool GMConfig::Load() {
@@ -282,6 +282,10 @@ GMConfig::deleg_db_t GMConfig::DelegationDBType() const {
   return deleg_db;
 }
 
+bool GMConfig::WLCGtoVOMS() const {
+  return wlcg_to_voms;
+}
+
 const std::string & GMConfig::ForcedVOMS(const char * queue) const {
   std::map<std::string,std::string>::const_iterator pos = forced_voms.find(queue);
   return (pos == forced_voms.end()) ? empty_string : pos->second;
@@ -301,6 +305,15 @@ const std::list<std::pair<bool,std::string> > & GMConfig::MatchingGroupsPublicIn
   return matching_groups_publicinfo;
 }
 
+const std::list<std::string> & GMConfig::TokenScopes(const char * action) const {
+  if(!action)
+    return empty_string_list;
+  std::map<std::string, std::list<std::string> >::const_iterator it = token_scopes.find(action);
+  if(it == token_scopes.end())
+    return empty_string_list;
+  return it->second;
+}
+
 bool GMConfig::Substitute(std::string& param, bool& userSubs, bool& otherSubs, const Arc::User& user) const {
   std::string::size_type curpos = 0;
   userSubs = false;
