@@ -191,8 +191,6 @@ public:
   bool EMIESInterfaceEnabled() const { return enable_emies_interface; }
   /// GridFTP job interface endpoint
   const std::string & GridFTPEndpoint() const { return gridftp_endpoint; }
-  /// Whether public information interface is enabled
-  bool PublicInformationEnabled() const { return enable_publicinfo; }
   /// A-REX WS-interface job submission endpoint
   const std::string & AREXEndpoint() const { return arex_endpoint; }
 
@@ -248,6 +246,8 @@ public:
   bool MatchShareUid(uid_t suid) const { return ((share_uid==0) || (share_uid==suid)); };
   /// Returns true if any of the shared gids matches the given gid
   bool MatchShareGid(gid_t sgid) const;
+  /// Returns true id token's claims from WLCG profile to be treated as VOMS attributes
+  bool WLCGtoVOMS() const;
   /// Returns forced VOMS attributes for users which have none.
   /// If queue is not specified value for server is returned.
   const std::string & ForcedVOMS(const char * queue = "") const;
@@ -258,6 +258,8 @@ public:
   const std::list<std::pair<bool,std::string> > & MatchingGroups(const char * queue = "") const;
   /// Returns list of authorization groups for public information.
   const std::list<std::pair<bool,std::string> > & MatchingGroupsPublicInformation() const;
+  /// Returns configured scopes for specified action.
+  const std::list<std::string> & TokenScopes(const char* action) const;
 
   bool UseSSH() const { return sshfs_mounts_enabled; }
   /// Check if remote directory is mounted
@@ -356,14 +358,14 @@ private:
   bool enable_arc_interface;
   /// Whether EMI-ES interface is enabled
   bool enable_emies_interface;
-  /// Whether public information interface is enabled
-  bool enable_publicinfo;
   /// GridFTP job endpoint
   std::string gridftp_endpoint;
   /// WS-interface endpoint
   std::string arex_endpoint;
   /// Delegation db type
   deleg_db_t deleg_db;
+  /// Either id token's claims from WLCG profile to be treated as VOMS attributes
+  bool wlcg_to_voms;
   /// Forced VOMS attribute for non-VOMS credentials per queue
   std::map<std::string,std::string> forced_voms;
   /// VOs authorized per queue
@@ -372,6 +374,7 @@ private:
   std::map<std::string, std::list<std::pair<bool, std::string> > > matching_groups;
   /// groups allowed to access public information with allow/deny mark (true/false)
   std::list<std::pair<bool, std::string> > matching_groups_publicinfo;
+  std::map<std::string, std::list<std::string> > token_scopes;
 
   /// Indicates whether session, runtime and cache dirs are mounted through sshfs (only suppored by Python backends) 
   bool sshfs_mounts_enabled;
