@@ -44,7 +44,6 @@ public:
   Arc::LogLevel get_log_level() const { return log_level; };
   std::string get_dtr_log() const { return dtr_log; };
   std::string get_dtr_central_log() const { return dtr_central_log; };
-  std::string get_acix_endpoint() const { return acix_endpoint; };
 
 private:
   /// Max transfers in delivery
@@ -92,9 +91,6 @@ private:
   /// Central log file for all DTR messages
   std::string dtr_central_log;
 
-  /// ACIX endpoint from which to find locations of cached files
-  std::string acix_endpoint;
-
   /// Validity of configuration
   bool valid;
 
