@@ -160,6 +160,11 @@ std::ostream &operator<< (std::ostream &o,const FileData &fd) {
       if(!escaped_cred.empty()) {
         o.put(' ');
         o.write(escaped_cred.c_str(), escaped_cred.size());
+        std::string escaped_cred_type(Arc::escape_chars(fd.cred_type, " \\\r\n", '\\', false));
+        if(!escaped_cred_type.empty()) {
+          o.put(' ');
+          o.write(escaped_cred_type.c_str(), escaped_cred_type.size());
+        };
       };
     };
   };
@@ -170,10 +175,11 @@ std::istream &operator>> (std::istream &i,FileData &fd) {
   std::string buf;
   std::getline(i,buf);
   Arc::trim(buf," \t\r\n");
-  fd.pfn.resize(0); fd.lfn.resize(0); fd.cred.resize(0);
-  fd.pfn  = Arc::unescape_chars(Arc::extract_escaped_token(buf, ' ', '\\'), '\\');
-  fd.lfn  = Arc::unescape_chars(Arc::extract_escaped_token(buf, ' ', '\\'), '\\');
-  fd.cred = Arc::unescape_chars(Arc::extract_escaped_token(buf, ' ', '\\'), '\\');
+  fd.pfn.resize(0); fd.lfn.resize(0); fd.cred.resize(0); fd.cred_type.resize(0);
+  fd.pfn       = Arc::unescape_chars(Arc::extract_escaped_token(buf, ' ', '\\'), '\\');
+  fd.lfn       = Arc::unescape_chars(Arc::extract_escaped_token(buf, ' ', '\\'), '\\');
+  fd.cred      = Arc::unescape_chars(Arc::extract_escaped_token(buf, ' ', '\\'), '\\');
+  fd.cred_type = Arc::unescape_chars(Arc::extract_escaped_token(buf, ' ', '\\'), '\\');
   if((fd.pfn.length() == 0) && (fd.lfn.length() == 0)) return i; /* empty st */
   if(!Arc::CanonicalDir(fd.pfn,true,true)) {
     logger.msg(Arc::ERROR,"Wrong directory in %s",buf);
@@ -289,6 +295,7 @@ JobLocalDescription& JobLocalDescription::operator=(const Arc::JobDescription& a
         // It is not possible to extract credentials path here.
         // So temporarily storing id here.
         inputdata.back().cred = file->Sources.front().DelegationID;
+        inputdata.back().cred_type = "id"; // just to have it set to something
       }
     }
 
@@ -347,6 +354,7 @@ JobLocalDescription& JobLocalDescription::operator=(const Arc::JobDescription& a
           // It is not possible to extract credentials path here.
           // So temporarily storing id here.
           outputdata.back().cred = target->DelegationID;
+          outputdata.back().cred_type = "id"; // just to have it set to something
         }
         if(outputdata.back().ifsuccess) ifsuccess = true;
         if(outputdata.back().ifcancel) ifcancel = true;
@@ -588,14 +596,20 @@ bool JobLocalDescription::write(const std::string& fname) const {
       vf != voms.end(); ++vf) {
     if(!write_pair(f,"voms",(*vf))) return false;
   };
+  for(std::list<std::string>::const_iterator ag=authgroups.begin();
+      ag != authgroups.end(); ++ag) {
+    if(!write_pair(f,"auth",(*ag))) return false;
+  };
   for(std::list<std::string>::const_iterator act_id=activityid.begin();
       act_id != activityid.end(); ++act_id) {
     if(!write_pair(f,"activityid",(*act_id))) return false;
   };
-  if (migrateactivityid != "") {
-    if(!write_pair(f,"migrateactivityid",migrateactivityid)) return false;
-    if(!write_pair(f,"forcemigration",forcemigration)) return false;
-  }
+  for(std::map<std::string, std::list<std::string> >::const_iterator claims=tokenclaim.begin();
+                                                               claims != tokenclaim.end(); ++claims) {
+    for(std::list<std::string>::const_iterator claim = claims->second.begin(); claim != claims->second.end(); ++claim) {
+      if(!write_pair(f,"tokenclaim."+claims->first,*claim)) return false;
+    };
+  };
   if(!write_pair(f,"transfershare",transfershare)) return false;
   if(!write_pair(f,"priority",Arc::tostring(priority))) return false;
   if(!write_pair(f,"dryrun",dryrun)) return false;
@@ -608,8 +622,10 @@ bool JobLocalDescription::read(const std::string& fname) {
   KeyValueFile f(fname,KeyValueFile::Fetch);
   if(!f) return false;
   activityid.clear();
+  tokenclaim.clear();
   localvo.clear();
   voms.clear();
+  authgroups.clear();
   for(;;) {
     std::string name;
     std::string buf;
@@ -710,6 +726,9 @@ bool JobLocalDescription::read(const std::string& fname) {
     else if(name == "voms") {
       voms.push_back(buf);
     }
+    else if(name == "auth") {
+      authgroups.push_back(buf);
+    }
     else if(name == "diskspace") {
       unsigned long long int n;
       if(!Arc::stringto(buf,n)) return false;
@@ -718,8 +737,6 @@ bool JobLocalDescription::read(const std::string& fname) {
     else if(name == "activityid") {
       activityid.push_back(buf);
     }
-    else if(name == "migrateactivityid") { migrateactivityid = buf; }
-    else if(name == "forcemigration") { forcemigration = parse_boolean(buf); }
     else if(name == "transfershare") { transfershare = buf; }
     else if(name == "priority") {
       int n;
@@ -727,6 +744,9 @@ bool JobLocalDescription::read(const std::string& fname) {
       priority = n;
     }
     else if(name == "dryrun") { dryrun = parse_boolean(buf); }
+    else if(name.compare(0,11,"tokenclaim.") == 0) {
+      tokenclaim[name.substr(11)].push_back(buf);
+    }
   }
   return true;
 }
