@@ -564,7 +564,7 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
   }
   
   // Print transfer statistics
-  std::string fname = config.ControlDir() + "/job." + job->get_id() + ".statistics";
+  std::string fname = job_control_path(config.ControlDir(),job->get_id(),sfx_statistics);
   std::ofstream f(fname.c_str(),std::ios::out | std::ios::app);
   if(f.is_open() ) {
     f << dtr_transfer_statistics << std::endl;  
@@ -689,6 +689,7 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
 
   // Default credentials to be used by transfering files if not specified per file
   std::string default_cred = job_proxy_filename(jobid, config); // TODO: drop job.proxy as source of delegation
+  std::string default_cred_type;
 
   JobLocalDescription job_desc;
   if(job_local_read_file(jobid, config, job_desc)) {
@@ -696,9 +697,11 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
       ARex::DelegationStores* delegs = config.GetDelegations();
       if(delegs) {
         DelegationStore& deleg = delegs->operator[](config.DelegationDir());
-        std::string fname = deleg.FindCred(job_desc.delegationid, job_desc.DN);
+	std::list<std::string> meta;
+        std::string fname = deleg.FindCred(job_desc.delegationid, job_desc.DN, meta);
         if(!fname.empty()) {
           default_cred = fname;
+	  if(!meta.empty()) default_cred_type = meta.front();
         }
       }
     }
@@ -707,7 +710,7 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
   DataStaging::DTRCredentialInfo cred_info(job_desc.DN, job_desc.expiretime, job_desc.voms);
 
   // Create a file for the transfer statistics and fix its permissions
-  std::string fname = config.ControlDir() + "/job." + jobid + ".statistics";
+  std::string fname = job_control_path(config.ControlDir(),jobid,sfx_statistics);
   std::ofstream f(fname.c_str(),std::ios::out | std::ios::app);
   f.close();
   fix_file_permissions(fname);
@@ -786,7 +789,11 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
         // there is a way to pass credentials for dynamic files. But so far default_cred
         // is always picked up.
         std::string cred(it->cred);
-        if(cred.empty()) cred = default_cred;
+	std::string cred_type(it->cred_type);
+        if(cred.empty()) {
+          cred = default_cred;
+          cred_type = default_cred_type;
+        }
         std::list<FileData> files_;
         std::string outputfilelist = job->SessionDir() + std::string("/") + it->pfn.substr(2);
         logger.msg(Arc::INFO, "%s: Reading output files from user generated list in %s", jobid, outputfilelist);
@@ -805,7 +812,10 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
         }
         // Attach dynamic files and assign credentials to them unless already available
         for(std::list<FileData>::iterator it_ = files_.begin(); it_ != files_.end(); ++it_) {
-          if(it_->cred.empty()) it_->cred = cred;
+          if(it_->cred.empty()) {
+            it_->cred = cred;
+            it_->cred_type = cred_type;
+          }
           files.push_back(*it_);
         }
         it->pfn.erase(1, 1);
@@ -815,6 +825,7 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
       if (it->pfn.rfind('/') == it->pfn.length()-1) {
         if (it->lfn.find(':') != std::string::npos) {
           std::string cred(it->cred);
+          std::string cred_type(it->cred_type);
           std::string dir(job->SessionDir() + it->pfn);
           std::list<std::string> entries;
           if (!Arc::DirList(dir, entries, true, job_uid, job_gid)) {
@@ -839,6 +850,7 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
               logger.msg(Arc::DEBUG, "%s: Adding new output file %s: %s", jobid, pfn, lfn);
               FileData fd(pfn, lfn);
               fd.cred = cred;
+              fd.cred_type = cred_type;
               files.push_back(fd);
             }
           }
@@ -914,8 +926,8 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
     logger.msg(Arc::WARNING, "%s: Session directory processing takes too long - %u.%06u seconds",
                                jobid, sessiondir_processing_time.GetPeriod(), sessiondir_processing_time.GetPeriodNanoseconds()/1000);
   }
-  Arc::initializeCredentialsType cred_type(Arc::initializeCredentialsType::SkipCredentials);
-  Arc::UserConfig usercfg(cred_type);
+  Arc::initializeCredentialsType cred_init_type(Arc::initializeCredentialsType::SkipCredentials);
+  Arc::UserConfig usercfg(cred_init_type);
   usercfg.UtilsDirPath(config.ControlDir());
   usercfg.CACertificatesDirectory(config.CertDir());
   if (config.StrictSession()) usercfg.SetUser(job->get_user());
@@ -942,30 +954,11 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
 
     staging = true;
     std::string source;
-    std::string original_source;
     std::string destination;
-    if (job->get_state() == JOB_STATE_PREPARING) {
-      // If ACIX should be used, use it as source and add original URL as
-      // location after DTR is created
-      // If cache=check then don't use remote caches as the copy must be up to date
-      Arc::URL original_url(i->lfn);
-      if (!staging_conf.get_acix_endpoint().empty() && original_url.Option("cache") != "check") {
-        original_source = i->lfn;
-        // Encode the original url so it is not parsed as part of the acix url
-        Arc::URL acix_source(staging_conf.get_acix_endpoint() + "?url=" + Arc::uri_encode(original_source, false));
-        // Add URL options to ACIX URL
-        for (std::map<std::string, std::string>::const_iterator opt = original_url.Options().begin();
-             opt != original_url.Options().end(); ++opt) {
-          acix_source.AddOption(opt->first, opt->second);
-        }
-        source = acix_source.fullstr();
-      }
-      else {
-        source = i->lfn;
-      }
+    if (job->get_state() == JOB_STATE_PREPARING) { // PREPARING
+      source = i->lfn;
       destination = "file:" + job->SessionDir() + i->pfn;
-    } // PREPARING
-    else { // FINISHING
+    } else { // FINISHING
       source = "file:" + job->SessionDir() + i->pfn;
       // Upload to dest ending in '/': append filename to lfn
       // Note: won't work for nested URLs used for index services
@@ -1001,11 +994,19 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
     }
     std::string proxy_cred;
     if(!i->cred.empty()) {
-      usercfg.ProxyPath(i->cred);
-      if (Arc::FileRead(i->cred, proxy_cred)) usercfg.CredentialString(proxy_cred);
+      if(i->cred_type.empty() || (i->cred_type == "x509")) {
+        usercfg.ProxyPath(i->cred);
+        if (Arc::FileRead(i->cred, proxy_cred)) usercfg.CredentialString(proxy_cred);
+      } else if(i->cred_type == "jwt") {
+        if (Arc::FileRead(i->cred, proxy_cred)) usercfg.OToken(proxy_cred);
+      }
     } else {
-      usercfg.ProxyPath(default_cred);
-      if (Arc::FileRead(default_cred, proxy_cred)) usercfg.CredentialString(proxy_cred);
+      if(default_cred_type.empty() || (default_cred_type == "x509")) {
+        usercfg.ProxyPath(default_cred);
+        if (Arc::FileRead(default_cred, proxy_cred)) usercfg.CredentialString(proxy_cred);
+      } else if(i->cred_type == "jwt") {
+        if (Arc::FileRead(default_cred, proxy_cred)) usercfg.OToken(proxy_cred);
+      }
     }
 
     std::list<DataStaging::DTRLogDestination> logs;
@@ -1034,11 +1035,6 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
       dtr->set_priority(job->GetLocalDescription(config)->priority);
     // set whether to use A-REX host certificate for remote delivery services
     dtr->host_cert_for_remote_delivery(staging_conf.use_host_cert_for_remote_delivery);
-    // set real location if ACIX is used
-    if (!original_source.empty()) {
-      dtr->get_source()->AddLocation(Arc::URL(original_source), Arc::URL(original_source).ConnectionURL());
-      dtr->set_use_acix(true);
-    }
     dtr->get_job_perf_log().SetOutput(staging_conf.perf_log.GetOutput());
     dtr->get_job_perf_log().SetEnabled(staging_conf.perf_log.GetEnabled());
 
