@@ -45,25 +45,32 @@ bool JobDescriptionHandler::process_job_req(const GMJob &job,JobLocalDescription
   // Add default credentials for file which have no own assigned.
   ARex::DelegationStores* delegs = config.GetDelegations();
   std::string default_cred = job_proxy_filename(job.get_id(), config); // TODO: drop job.proxy as source of delegation
+  std::string default_cred_type;
   if(!job_desc.delegationid.empty()) {
     if(delegs) {
+      std::list<std::string> meta;
       DelegationStore& deleg = delegs->operator[](config.DelegationDir());
-      std::string fname = deleg.FindCred(job_desc.delegationid, job_desc.DN);
+      std::string fname = deleg.FindCred(job_desc.delegationid, job_desc.DN, meta);
       if(!fname.empty()) {
         default_cred = fname;
+        default_cred_type = (!meta.empty())?meta.front():"";
       };
     };
   };
 
+  // Resolve delegation ids into proxy credential paths
   for(std::list<FileData>::iterator f = job_desc.inputdata.begin();
                                    f != job_desc.inputdata.end(); ++f) {
     if(f->has_lfn()) {
       if(f->cred.empty()) {
         f->cred = default_cred;
+        f->cred_type = default_cred_type;
       } else {
         std::string path;
-        if(delegs) path = (*delegs)[config.DelegationDir()].FindCred(f->cred,job_desc.DN);
+        std::list<std::string> meta;
+        if(delegs) path = (*delegs)[config.DelegationDir()].FindCred(f->cred,job_desc.DN,meta);
         f->cred = path;
+        f->cred_type = (!meta.empty())?meta.front():"";
       };
     };
   };
@@ -74,9 +81,11 @@ bool JobDescriptionHandler::process_job_req(const GMJob &job,JobLocalDescription
         f->cred = default_cred;
       } else {
         std::string path;
+        std::list<std::string> meta;
         ARex::DelegationStores* delegs = config.GetDelegations();
-        if(delegs) path = (*delegs)[config.DelegationDir()].FindCred(f->cred,job_desc.DN);
+        if(delegs) path = (*delegs)[config.DelegationDir()].FindCred(f->cred,job_desc.DN,meta);
         f->cred = path;
+        f->cred_type = (!meta.empty())?meta.front():"";
       };
     };
   };
@@ -85,14 +94,36 @@ bool JobDescriptionHandler::process_job_req(const GMJob &job,JobLocalDescription
   return true;
 }
 
-JobReqResult JobDescriptionHandler::parse_job_req(JobLocalDescription &job_desc,Arc::JobDescription& arc_job_desc,const std::string &fname,bool check_acl) const {
+JobReqResult JobDescriptionHandler::parse_job_req_from_mem(JobLocalDescription &job_desc,Arc::JobDescription& arc_job_desc,const std::string &desc_str,bool check_acl) const {
+  {
+    std::list<Arc::JobDescription> descs;
+    Arc::JobDescriptionResult r = Arc::JobDescription::Parse(desc_str, descs, "", "GRIDMANAGER");
+    if (!r) {
+      std::string failure = r.str();
+      if(failure.empty()) failure = "Unable to parse job description.";
+      return JobReqResult(JobReqInternalFailure, "", failure);
+    }
+    if(descs.size() != 1) {
+      return JobReqResult(JobReqInternalFailure, "", "Multiple job descriptions not supported");
+    }
+    arc_job_desc = descs.front();
+  }
+
+  return parse_job_req_internal(job_desc, arc_job_desc, check_acl);
+}
+
+JobReqResult JobDescriptionHandler::parse_job_req_from_file(JobLocalDescription &job_desc,Arc::JobDescription& arc_job_desc,const std::string &fname,bool check_acl) const {
   Arc::JobDescriptionResult arc_job_res = get_arc_job_description(fname, arc_job_desc);
   if (!arc_job_res) {
     std::string failure = arc_job_res.str();
     if(failure.empty()) failure = "Unable to read or parse job description.";
     return JobReqResult(JobReqInternalFailure, "", failure);
   }
+  return parse_job_req_internal(job_desc, arc_job_desc, check_acl);
+}
 
+
+JobReqResult JobDescriptionHandler::parse_job_req_internal(JobLocalDescription &job_desc,Arc::JobDescription const& arc_job_desc,bool check_acl) const {
   if (!arc_job_desc.Resources.RunTimeEnvironment.isResolved()) {
     return JobReqResult(JobReqInternalFailure, "", "Runtime environments have not been resolved.");
   }
@@ -145,14 +176,14 @@ JobReqResult JobDescriptionHandler::parse_job_req(const JobId &job_id,JobLocalDe
 }
 
 JobReqResult JobDescriptionHandler::parse_job_req(const JobId &job_id,JobLocalDescription &job_desc,Arc::JobDescription& arc_job_desc,bool check_acl) const {
-  std::string fname = config.ControlDir() + "/job." + job_id + ".description";
-  return parse_job_req(job_desc,arc_job_desc,fname,check_acl);
+  std::string fname = job_control_path(config.ControlDir(),job_id,sfx_desc);
+  return parse_job_req_from_file(job_desc,arc_job_desc,fname,check_acl);
 }
 
 std::string JobDescriptionHandler::get_local_id(const JobId &job_id) const {
   std::string id;
   std::string joboption("joboption_jobid=");
-  std::string fgrami(config.ControlDir() + "/job." + job_id + ".grami");
+  std::string fgrami(job_control_path(config.ControlDir(),job_id,sfx_grami));
   std::list<std::string> grami_data;
   if (Arc::FileRead(fgrami, grami_data)) {
     for (std::list<std::string>::iterator line = grami_data.begin(); line != grami_data.end(); ++line) {
@@ -182,7 +213,7 @@ bool JobDescriptionHandler::write_grami_executable(std::ofstream& f, const std::
 }
 
 bool JobDescriptionHandler::write_grami(GMJob &job,const char *opt_add) const {
-  const std::string fname = config.ControlDir() + "/job." + job.get_id() + ".description";
+  const std::string fname = job_control_path(config.ControlDir(),job.get_id(),sfx_desc);
 
   Arc::JobDescription arc_job_desc;
   if (!get_arc_job_description(fname, arc_job_desc)) return false;
@@ -195,7 +226,7 @@ bool JobDescriptionHandler::write_grami(const Arc::JobDescription& arc_job_desc,
   JobLocalDescription& job_local_desc = *(job.GetLocalDescription(config));
   const std::string session_dir = job.SessionDir();
   const std::string control_dir = config.ControlDir();
-  const std::string fgrami = control_dir + "/job." + job.get_id() + ".grami";
+  const std::string fgrami = job_control_path(control_dir,job.get_id(),sfx_grami);
   std::ofstream f(fgrami.c_str(),std::ios::out | std::ios::trunc);
   if(!f.is_open()) return false;
   if(!fix_file_permissions(fgrami,job,config)) return false;
@@ -382,7 +413,7 @@ JobReqResult JobDescriptionHandler::get_acl(const Arc::JobDescription& arc_job_d
 
 /* parse job description and set specified file permissions to executable */
 bool JobDescriptionHandler::set_execs(const GMJob &job) const {
-  std::string fname = config.ControlDir() + "/job." + job.get_id() + ".description";
+  std::string fname = job_control_path(config.ControlDir(),job.get_id(),sfx_desc);
   Arc::JobDescription desc;
   if (!get_arc_job_description(fname, desc)) return false;
 
