@@ -281,8 +281,8 @@ bool JobsList::ScanOldJobs(void) {
     // todo: implement fetching few jobs before passing them to attention
     // job id must contain at least one character
     int l=file.length();
-    if(l>(4+7) && file.substr(0,4) == "job." && file.substr(l-7) == ".status") {
-      JobId id(file.substr(4, l-7-4));
+    if(l>7 && file.substr(l-7) == ".status") {
+      JobId id(file.substr(0, l-7));
       logger.msg(Arc::DEBUG, "%s: job found while scanning", id);
       RequestAttention(id);
     };
@@ -417,17 +417,32 @@ bool JobsList::FailedJob(GMJobRef i,bool cancel) {
     r = false;
   }
   // Convert delegation ids to credential paths.
-  std::string default_cred = config.ControlDir() + "/job." + i->get_id() + ".proxy";
+  ARex::DelegationStores* delegs = config.GetDelegations();
+  std::string default_cred = job_proxy_filename(i->get_id(), config); // TODO: drop job.proxy as source of delegation
+  std::string default_cred_type;
+  if(!job_desc.delegationid.empty()) {
+    if(delegs) {
+      std::list<std::string> meta;
+      DelegationStore& deleg = delegs->operator[](config.DelegationDir());
+      std::string fname = deleg.FindCred(job_desc.delegationid, job_desc.DN, meta);
+      if(!fname.empty()) {
+        default_cred = fname;
+        default_cred_type = (!meta.empty())?meta.front():"";
+      };
+    };
+  };
   for(std::list<FileData>::iterator f = job_desc.outputdata.begin();
                                    f != job_desc.outputdata.end(); ++f) {
     if(f->has_lfn()) {
       if(f->cred.empty()) {
         f->cred = default_cred;
+        f->cred_type = default_cred_type;
       } else {
         std::string path;
-        ARex::DelegationStores* delegs = config.GetDelegations();
-        if(delegs && i->local) path = (*delegs)[config.DelegationDir()].FindCred(f->cred,i->local->DN);
+        std::list<std::string> meta;
+        if(delegs && i->local) path = (*delegs)[config.DelegationDir()].FindCred(f->cred,i->local->DN,meta);
         f->cred = path;
+	f->cred_type = (!meta.empty())?meta.front():"";
       }
       if(i->local) ++(i->local->uploads);
     }
@@ -533,7 +548,7 @@ bool JobsList::state_submitting(GMJobRef i,bool &state_changed) {
     // submit job to LRMS using submit-X-job
     std::string cmd = Arc::ArcLocation::GetDataDir()+"/submit-"+job_desc->lrms+"-job";
     logger.msg(Arc::INFO,"%s: state SUBMIT: starting child: %s",i->job_id,cmd);
-    std::string grami = config.ControlDir()+"/job."+(*i).job_id+".grami";
+    std::string grami = job_control_path(config.ControlDir(),(*i).job_id,sfx_grami);
     cmd += " --config " + config.ConfigFile() + " " + grami;
     job_errors_mark_put(*i,config);
     i->child_output.clear();
@@ -642,7 +657,7 @@ bool JobsList::state_canceling(GMJobRef i,bool &state_changed) {
       state_changed=true;
       return true;
     }
-    std::string grami = config.ControlDir()+"/job."+(*i).job_id+".grami";
+    std::string grami = job_control_path(config.ControlDir(),(*i).job_id,sfx_grami);
     cmd += " --config " + config.ConfigFile() + " " + grami;
     job_errors_mark_put(*i,config);
     if(!RunParallel::run(config,*i,*this,NULL,cmd,&(i->child))) {
@@ -1471,69 +1486,67 @@ bool JobsList::ActJob(GMJobRef& i) {
 
   if (job_result != JobDropped) {
 
-  if(old_state != i->job_state) {
-    // Job changed state. Skip state change processing if job is about to be dropped.
-    if(!job_state_write_file(*i,config,i->job_state,i->job_pending)) {
-      i->AddFailure("Failed writing job status: "+Arc::StrError(errno));
-      job_result = ActJobFailed(i); // immedaitely process failure
-    } else {
-      // Talk to external plugin to ask if we can proceed
-      // Jobs with ACCEPTED state or UNDEFINED previous state
-      // could be ignored here. But there is tiny possibility
-      // that service failed while processing ContinuationPlugins.
-      // Hence here we have duplicate call for ACCEPTED state.
-      // TODO: maybe introducing job state prefix VALIDATING:
-      // could be used to resolve this situation.
-      if(!CheckJobContinuePlugins(i)) {
-        // No need for AddFailure. It is filled inside CheckJobContinuePlugins.
+    if(old_state != i->job_state) {
+      // Job changed state. Skip state change processing if job is about to be dropped.
+      if(!job_state_write_file(*i,config,i->job_state,i->job_pending)) {
+        i->AddFailure("Failed writing job status: "+Arc::StrError(errno));
         job_result = ActJobFailed(i); // immedaitely process failure
+      } else {
+        // Talk to external plugin to ask if we can proceed
+        // Jobs with ACCEPTED state or UNDEFINED previous state
+        // could be ignored here. But there is tiny possibility
+        // that service failed while processing ContinuationPlugins.
+        // Hence here we have duplicate call for ACCEPTED state.
+        // TODO: maybe introducing job state prefix VALIDATING:
+        // could be used to resolve this situation.
+        if(!CheckJobContinuePlugins(i)) {
+          // No need for AddFailure. It is filled inside CheckJobContinuePlugins.
+          job_result = ActJobFailed(i); // immedaitely process failure
+        };
+        // Processing to be done on relatively successful state changes
+        JobLog* joblog = config.GetJobLog();
+        if(joblog) joblog->WriteJobRecord(*i,config);
+        // TODO: Consider moving following code into ActJob* methods
+        if(i->job_state == JOB_STATE_FINISHED) {
+          job_clean_finished(i->job_id,config);
+          if(joblog) joblog->WriteFinishInfo(*i,config);
+          PrepareCleanupTime(i,i->keep_finished);
+        } else if(i->job_state == JOB_STATE_PREPARING) {
+          joblog->WriteStartInfo(*i,config);
+        };
       };
-      // Processing to be done on relatively successful state changes
-      JobLog* joblog = config.GetJobLog();
-      if(joblog) joblog->WriteJobRecord(*i,config);
-      // TODO: Consider moving following code into ActJob* methods
-      if(i->job_state == JOB_STATE_FINISHED) {
-        job_clean_finished(i->job_id,config);
-        if(joblog) joblog->WriteFinishInfo(*i,config);
-        PrepareCleanupTime(i,i->keep_finished);
-      } else if(i->job_state == JOB_STATE_PREPARING) {
-        joblog->WriteStartInfo(*i,config);
-      };
-//            SetJobState(i, JOB_STATE_FINISHED, "Job processing error");
-//            SetJobState(i, JOB_STATE_FINISHING, "Job processing error");
-    };
-    // send mail after error and change are processed
-    // do not send if something really wrong happened to avoid email DoS
-    if(job_result != JobFailed) send_mail(*i,config);
+      // send mail after error and change are processed
+      // do not send if something really wrong happened to avoid email DoS
+      if(job_result != JobFailed) send_mail(*i,config);
 
-    // Manage per-DN counter
-    // Any job state change goes through here
-    if(!IS_ACTIVE_STATE(old_state)) {
-      if(IS_ACTIVE_STATE(i->job_state)) {
-        if(i->GetLocalDescription(config)) {
-          // add to DN map
-          if (i->local->DN.empty()) {
-             logger.msg(Arc::WARNING, "Failed to get DN information from .local file for job %s", i->job_id);
-          }
-          Glib::RecMutex::Lock lock(jobs_lock);
-          ++(jobs_dn[i->local->DN]);
+      // Manage per-DN counter
+      // Any job state change goes through here
+      if(!IS_ACTIVE_STATE(old_state)) {
+        if(IS_ACTIVE_STATE(i->job_state)) {
+          if(i->GetLocalDescription(config)) {
+            // add to DN map
+            if (i->local->DN.empty()) {
+              logger.msg(Arc::WARNING, "Failed to get DN information from .local file for job %s", i->job_id);
+            }
+            Glib::RecMutex::Lock lock(jobs_lock);
+            ++(jobs_dn[i->local->DN]);
+          };
         };
-      };
-    } else if(IS_ACTIVE_STATE(old_state)) {
-      if(!IS_ACTIVE_STATE(i->job_state)) {
-        if(i->GetLocalDescription(config)) {
-          Glib::RecMutex::Lock lock(jobs_lock);
-          if (--(jobs_dn[i->local->DN]) == 0) jobs_dn.erase(i->local->DN);
+      } else if(IS_ACTIVE_STATE(old_state)) {
+        if(!IS_ACTIVE_STATE(i->job_state)) {
+          if(i->GetLocalDescription(config)) {
+            Glib::RecMutex::Lock lock(jobs_lock);
+            if (--(jobs_dn[i->local->DN]) == 0) jobs_dn.erase(i->local->DN);
+          };
         };
       };
+    } else if(old_pending != i->job_pending) {
+      // Only pending flag has changed - only write state file
+      if(!job_state_write_file(*i,config,i->job_state,i->job_pending)) {
+        i->AddFailure("Failed writing job status: "+Arc::StrError(errno));
+        job_result = ActJobFailed(i); // immedaitely process failure
+      }
     };
-  } else if(old_pending != i->job_pending) {
-    // Only pending flag has changed - only write state file
-    if(!job_state_write_file(*i,config,i->job_state,i->job_pending)) {
-      i->AddFailure("Failed writing job status: "+Arc::StrError(errno));
-      job_result = ActJobFailed(i); // immedaitely process failure
-    }
-  };
 
   }; // !JobDropped
 
@@ -1589,6 +1602,13 @@ JobsList::ActJobResult JobsList::ActJobFailed(GMJobRef i) {
       // No matter if FINISHING fails - it still goes to FINISHED
       SetJobState(i, JOB_STATE_FINISHED, "Job failure detected");
       RequestReprocess(i);
+    } else if(i->job_state == JOB_STATE_INLRMS) {
+      // This happens either if job processing failed or continuation
+      // plugin failed. But that also means job is probably being
+      // processed by batch system. So safest is to act as if cncel 
+      // request arrived.
+      SetJobState(i, JOB_STATE_CANCELING, "Job failure detected");
+      RequestReprocess(i);
     } else {
       // Other states are moved to FINISHING and start post-staging
       SetJobState(i, JOB_STATE_FINISHING, "Job failure detected");
@@ -1620,7 +1640,7 @@ bool JobsList::RestartJobs(const std::string& cdir,const std::string& odir) {
       if(file.empty()) break;
       int l=file.length();
       // job id contains at least 1 character
-      if(l>(4+7) && file.substr(0,4) == "job." && file.substr(l-7) == ".status") {
+      if(l>7 && file.substr(l-7) == ".status") {
         uid_t uid;
         gid_t gid;
         time_t t;
@@ -1654,7 +1674,7 @@ bool JobsList::RestartJobs(void) {
 
 bool JobsList::ScanJobDesc(const std::string& cdir, JobFDesc& id) {
   if(!FindJob(id.id)) {
-    std::string fname=cdir+'/'+"job."+id.id+".status";
+    std::string fname=cdir+'/'+id.id+"."+sfx_status;
     uid_t uid;
     gid_t gid;
     time_t t;
@@ -1690,8 +1710,8 @@ bool JobsList::ScanAllJobs(const std::string& cdir,std::list<JobFDesc>& ids, Job
       if(file.empty()) break;
       int l=file.length();
       // job id contains at least 1 character
-      if(l>(4+7) && file.substr(0,4) == "job." && file.substr(l-7) == ".status") {
-        JobFDesc id(file.substr(4,l-7-4));
+      if(l>7 && file.substr(l-7) == ".status") {
+        JobFDesc id(file.substr(0,l-7));
         if(filter.accept(id.id)) {
           std::string fname=cdir+'/'+file.c_str();
           uid_t uid;
@@ -1722,13 +1742,13 @@ bool JobsList::ScanMarks(const std::string& cdir,const std::list<std::string>& s
       std::string file=dir.read_name();
       if(file.empty()) break;
       int l=file.length();
-      // job id contains at least 1 character
-      if(l>(4+7) && file.substr(0,4) == "job.") {
+      // job id contains at least 1 separtor and 1 suffix character
+      if(l>1) {
         for(std::list<std::string>::const_iterator sfx = suffices.begin();
             sfx != suffices.end();++sfx) {
           int ll = sfx->length();
-          if(l > (ll+4) && file.substr(l-ll) == *sfx) {
-            JobFDesc id(file.substr(4,l-ll-4));
+          if(l > ll && file.substr(l-ll) == *sfx) {
+            JobFDesc id(file.substr(0,l-ll));
             if(!FindJob(id.id)) {
               std::string fname=cdir+'/'+file.c_str();
               uid_t uid;
@@ -1921,7 +1941,7 @@ GMJobRef JobsList::GetJob(const GMConfig& config, const JobId& id) {
                                subdir != subdirs.end();++subdir) {
     std::string cdir=config.ControlDir();
     std::string odir=cdir+(*subdir);
-    std::string fname=odir+'/'+"job."+id+".status";
+    std::string fname=odir+'/'+id+".status";
     uid_t uid;
     gid_t gid;
     time_t t;
