@@ -38,6 +38,11 @@ common_init () {
     [ -e "${pkgdatadir}/community_rtes.sh" ] && . "${pkgdatadir}/community_rtes.sh"
 }
 
+# defines failures_file
+define_failures_file () {
+    failures_file=$(control_path "$joboption_controldir" "$joboption_gridid" "failed")
+}
+
 # checks any scratch is defined (shared or local)
 check_any_scratch () {
     if [ -z "${RUNTIME_NODE_SEES_FRONTEND}" ] ; then
@@ -575,8 +580,9 @@ clean_local_scratch_dir_output () {
   echo '  find ./ -type l -exec rm -f "{}" ";"' >> $LRMS_JOB_SCRIPT
   echo '  chmod -R u+w "./"' >> $LRMS_JOB_SCRIPT
   
-  if [ -f "$joboption_controldir/job.$joboption_gridid.output" ] ; then
-    cat "$joboption_controldir/job.$joboption_gridid.output" | \
+  output_file=$(control_path "$joboption_controldir" "$joboption_gridid" "output")
+  if [ -f "$output_file" ] ; then
+    cat "$output_file" | \
     # remove leading backslashes, if any
     sed 's/^\/*//' | \
     # backslashes and spaces are escaped with a backslash in job.*.output. The
@@ -719,7 +725,8 @@ EOSCR
   fi
   echo "echo \"Benchmark=$joboption_benchmark\" >> \"\$RUNTIME_JOB_DIAG\"" >> $LRMS_JOB_SCRIPT
   # add queue benchmark to frontend diag (for jobs that failed to reach/start in LRMS)
-  echo "Benchmark=$joboption_benchmark" >> "${joboption_controldir}/job.${joboption_gridid}.diag"
+  diag_file=$(control_path "$joboption_controldir" "$joboption_gridid" "diag")
+  echo "Benchmark=$joboption_benchmark" >> "${diag_file}"
 
   # Define executable and check it exists on the worker node
   echo "executable='$joboption_arg_0'" >> $LRMS_JOB_SCRIPT
