@@ -1,7 +1,8 @@
 from __future__ import print_function
 from __future__ import absolute_import
 
-from .AccountingDB import AccountingDB, AAR
+from .ControlCommon import HTTPSClientAuthConnection, print_info
+from .AccountingDB import AccountingDB, AAR, ACCOUNTING_DB_FILE
 from arc.paths import ARC_VERSION, ARC_RUN_DIR
 
 import os
@@ -11,14 +12,6 @@ import re
 import datetime
 import calendar
 
-# SGASSender and AMSDirectSender dependencies:
-import socket
-import ssl
-try:
-    import httplib
-except ImportError:
-    import http.client as httplib
-
 # AMSDirectSender dependencies
 import json
 import base64
@@ -32,6 +25,8 @@ except ImportError:
     except ImportError:
         import StringIO
 
+ACCOUNTING_PUBLISHING_DB_FILE = "publishing.db"
+
 # module regexes init
 __voms_fqan_re = re.compile(r'(?P<group>/[-\w.]+(?:/(?!Role=)[-\w.]+)*)(?P<role>/Role=[-\w.]+)?(?P<cap>/Capability=[-\w.]+)?')
 
@@ -117,25 +112,25 @@ def get_apel_benchmark(logger, benchmark):
 #
 class RecordsPublisher(object):
     """Class for publishing accounting records to central network services"""
-    def __init__(self, arcconfig):
+    def __init__(self, arcconfig, adb=None):
         self.logger = logging.getLogger('ARC.Accounting.Publisher')
-        self.adb = None
         # arc config
         if arcconfig is None:
             self.logger.error('Failed to parse arc.conf. Accounting publishing is not possible.')
             sys.exit(1)
         self.arcconfig = arcconfig
         # get configured accounting targets and options
-        self.vomsless_vo = self.arcconfig.get_value('vomsless_vo', ['arex/jura'])
         self.extra_vogroups = self.arcconfig.get_value('vo_group', ['arex/jura'], force_list=True)
         self.conf_targets = list(map(lambda t: ('sgas', t), self.arcconfig.get_subblocks('arex/jura/sgas')))
         self.conf_targets += list(map(lambda t: ('apel', t), self.arcconfig.get_subblocks('arex/jura/apel')))
         # accounting database files and connection
         self.accounting_dir = arcconfig.get_value('controldir', 'arex').rstrip('/') + '/accounting'
-        adb_file = self.accounting_dir + '/accounting.db'
-        self.adb = AccountingDB(adb_file)
+        self.adb = adb
+        if self.adb is None:
+            adb_file = os.path.join(self.accounting_dir, ACCOUNTING_DB_FILE)
+            self.adb = AccountingDB(adb_file)
         # publishing state database file
-        self.pdb_file = self.accounting_dir + '/publishing.db'
+        self.pdb_file = os.path.join(self.accounting_dir, ACCOUNTING_PUBLISHING_DB_FILE)
         self.logger.debug('Accounting records publisher had been initialized')
 
     def __del__(self):
@@ -189,7 +184,7 @@ class RecordsPublisher(object):
         # only finished jobs
         self.adb.filter_statuses(['completed', 'failed'])
 
-    def publish_sgas(self, target_conf, endfrom, endtill=None):
+    def publish_sgas(self, target_conf, endfrom, endtill=None, dry_run=False):
         """Publish to SGAS target"""
         self.logger.debug('Assigning filters to SGAS publishing database query')
         self.__init_adb_filters(endfrom, endtill)
@@ -216,6 +211,7 @@ class RecordsPublisher(object):
             target_conf[k] = self.arcconfig.get_value(k, ['arex/jura'])
         # init SGAS sender
         sgassender = SGASSender(target_conf)
+        sgassender.set_dry_run(dry_run)
         # get AARs from database
         self.logger.debug('Querying AARS from accounting database')
         aars = self.adb.get_aars(resolve_ids=True)
@@ -227,7 +223,7 @@ class RecordsPublisher(object):
         # build URs
         self.logger.debug('Going to create OGF.98 Usage Records based on the AARs')
         urs = [UsageRecord(aar, localid_prefix=localid_prefix,
-                           vomsless_vo=self.vomsless_vo, extra_vogroups=self.extra_vogroups) for aar in aars]
+                           extra_vogroups=self.extra_vogroups) for aar in aars]
         if not urs:
             self.logger.error('Failed to build OGF.98 Usage Records for SGAS publishing')
             return None
@@ -239,7 +235,7 @@ class RecordsPublisher(object):
         self.logger.debug('Accounting records have been published to SGAS target %s', target_conf['targethost'])
         return latest_endtime
 
-    def publish_apel(self, target_conf, endfrom, endtill=None, regular=False):
+    def publish_apel(self, target_conf, endfrom, endtill=None, regular=False, dry_run=False):
         """Publish to all APEL targets"""
         # Parse targetURL and get necessary parameters for APEL publishing
         urldict = get_url_components(target_conf['targeturl'])
@@ -252,6 +248,7 @@ class RecordsPublisher(object):
         os.environ['RANDFILE'] = ARC_RUN_DIR + '/openssl.rnd'
         # init APEL AMS sender
         apel_sender = APELAMSDirectSender(target_conf)
+        apel_sender.set_dry_run(dry_run)
         # database query filters
         self.logger.debug('Assigning filters to APEL usage records database query.')
         self.__init_adb_filters(endfrom, endtill)
@@ -276,7 +273,7 @@ class RecordsPublisher(object):
                 # create EMI CAR URs
                 self.logger.debug('Going to create EMI CAR v1.2 Compute Accounting Records based on the AARs')
                 cars = [ComputeAccountingRecord(aar, gocdb_name=target_conf['gocdb_name'],
-                               vomsless_vo=self.vomsless_vo, extra_vogroups=self.extra_vogroups) for aar in aars]
+                                                extra_vogroups=self.extra_vogroups) for aar in aars]
                 if not cars:
                     self.logger.error('Failed to build EMI CAR v1.2 records for APEL publishing.')
                     return None
@@ -314,7 +311,24 @@ class RecordsPublisher(object):
                 apel_sender.publish_summaries(apelsummaries)
         # Sync messages are always sent and contains job counts for all periods
         self.logger.debug('Assigning filters to APEL sync database query')
-        self.__init_adb_filters()
+        sync_endtill = None
+        # Sync interval
+        if regular:
+            endfrom_year = endfrom.year - 1
+            sync_endfrom = endfrom.replace(day=1, year=endfrom_year, hour=0, minute=0, second=0)
+            self.logger.debug('Querying sync data for the last year')
+        else:
+            sync_endfrom = endfrom.replace(day=1, hour=0, minute=0, second=0)
+            if endtill:
+                endtillx = endtill - datetime.timedelta(seconds=1)
+                endtill_month = endtillx.month + 1
+                endtill_year = endtillx.year
+                if endtill_month == 13:
+                        endtill_year += 1
+                        endtill_month = 1
+                sync_endtill = endtillx.replace(day=1, month=endtill_month, year=endtill_year, hour=0, minute=0, second=0)
+            self.logger.debug('Querying sync data for the republished data months')
+        self.__init_adb_filters(sync_endfrom, sync_endtill)
         # optional WLCG VO filtering
         self.__add_vo_filter(target_conf)
         self.logger.debug('Querying APEL Sync data from accounting database')
@@ -337,7 +351,7 @@ class RecordsPublisher(object):
                 return targetconf, ttype
         return None, None
 
-    def republish(self, targettype, targetconf, endfrom, endtill=None):
+    def republish(self, targettype, targetconf, endfrom, endtill=None, dry_run=False):
         self.logger.info('Starting republishing process to %s target', targettype.upper())
         # check config for mandatory options
         if not self.__check_target_confdict(targettype, targetconf):
@@ -345,9 +359,9 @@ class RecordsPublisher(object):
         # publish without recording the last intervals
         latest = None
         if targettype == 'apel':
-            latest = self.publish_apel(targetconf, endfrom, endtill)
+            latest = self.publish_apel(targetconf, endfrom, endtill, dry_run=dry_run)
         elif targettype == 'sgas':
-            latest = self.publish_sgas(targetconf, endfrom, endtill)
+            latest = self.publish_sgas(targetconf, endfrom, endtill, dry_run=dry_run)
         if latest is None:
             return False
         return True
@@ -390,51 +404,18 @@ class RecordsPublisher(object):
                                  'have been successfully published to [%s] target',
                                  datetime.datetime.utcfromtimestamp(latest), target)
 
-
-class HTTPSClientAuthConnection(httplib.HTTPSConnection):
-    """ Class to make a HTTPS connection, with support for full client-based SSL Authentication"""
-
-    def __init__(self, host, port, key_file, cert_file, cacerts_path=None, timeout=None):
-        httplib.HTTPSConnection.__init__(self, host, port, key_file=key_file, cert_file=cert_file)
-        self.key_file = key_file
-        self.cert_file = cert_file
-        self.cacerts_path = cacerts_path
-        self.timeout = timeout
-
-    def connect(self):
-        """ Connect to a host on a given (SSL) port.
-            If ca_certs_path is pointing somewhere, use it to check Server Certificate.
-            This is needed to pass ssl.CERT_REQUIRED as parameter to SSLContext for Python 2.6+
-            which forces SSL to check server certificate against CA certificates in the defined location
-        """
-        sock = socket.create_connection((self.host, self.port), self.timeout)
-        if self._tunnel_host:
-            self.sock = sock
-            self._tunnel()
-        if self.cacerts_path and hasattr(ssl, 'SSLContext'):
-            protocol = ssl.PROTOCOL_SSLv23
-            if hasattr(ssl, 'PROTOCOL_TLS'):
-                protocol = ssl.PROTOCOL_TLS
-            # create SSL context with CA certificates verification
-            ssl_ctx = ssl.SSLContext(protocol)
-            ssl_ctx.load_verify_locations(capath=self.cacerts_path)
-            ssl_ctx.verify_mode = ssl.CERT_REQUIRED
-            ssl_ctx.load_cert_chain(self.cert_file, self.key_file)
-            ssl_ctx.check_hostname = True
-            self.sock = ssl_ctx.wrap_socket(sock, server_hostname=self.host)
-        else:
-            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
-                                        cert_reqs=ssl.CERT_NONE)
-
-
 class SGASSender(object):
     """Send messages to SGAS server"""
     def __init__(self, targetconf):
         self.logger = logging.getLogger('ARC.Accounting.SGAS')
+        self.dry_run = False
         self.conf = targetconf
         self.batchsize = int(self.conf['urbatchsize']) if 'urbatchsize' in self.conf else 50
         self.logger.debug('Initializing SGAS records sender for %s', self.conf['targethost'])
 
+    def set_dry_run(self, enable=True):
+        self.dry_run = enable
+
     def __post_xml(self, xmldata):
         self.logger.debug('Initializing HTTPS connection to %s:%s', self.conf['targethost'], self.conf['targetport'])
         conn = HTTPSClientAuthConnection(
@@ -475,7 +456,12 @@ class SGASSender(object):
             buf.write(UsageRecord.batch_header())
             buf.write(UsageRecord.join_str().join(map(lambda ur: ur.get_xml(), urs[x:x + self.batchsize])))
             buf.write(UsageRecord.batch_footer())
-            published = self.__post_xml(buf.getvalue())
+            if not self.dry_run:
+                published = self.__post_xml(buf.getvalue())
+            else:
+                print_info(self.logger, 'Printing the content of JobUsageRecords insted of sending it to SGAS')
+                print(buf.getvalue())
+                published = True
             buf.close()
             del buf
             if not published:
@@ -487,6 +473,7 @@ class APELAMSDirectSender(object):
     """Send messages to APEL via AMS REST interface direct usage"""
     def __init__(self, targetconf):
         self.logger = logging.getLogger('ARC.Accounting.AMS')
+        self.dry_run = False
         self.conf = targetconf
         self.batchsize = int(self.conf['urbatchsize']) if 'urbatchsize' in self.conf else 500
         # AMS project
@@ -500,6 +487,9 @@ class APELAMSDirectSender(object):
         self.publish_error = False
         self.logger.debug('Initializing APEL publishing via AMS REST for %s', self.conf['targethost'])
 
+    def set_dry_run(self, enable=True):
+        self.dry_run = enable
+
     def _obtain_ams_token(self):
         if self.ams_token is not None:
             return True
@@ -612,8 +602,12 @@ class APELAMSDirectSender(object):
             buf.write(ComputeAccountingRecord.join_str().join(map(lambda ur: ur.get_xml(),
                                                                   carlist[x:x + self.batchsize])))
             buf.write(ComputeAccountingRecord.batch_footer())
-            if not self._msg_send(buf.getvalue()):
-                self.publish_error = True
+            if self.dry_run:
+                print_info(self.logger, 'Printing the content of the CAR instead of sending it to APEL')
+                print(buf.getvalue())
+            else:
+                if not self._msg_send(buf.getvalue()):
+                    self.publish_error = True
             buf.close()
             del buf
 
@@ -624,8 +618,12 @@ class APELAMSDirectSender(object):
             buf.write(summaries[0].header())
             buf.write(summaries[0].join_str().join(map(lambda s: s.get_record(), summaries[x:x + self.batchsize])))
             buf.write(summaries[0].footer())
-            if not self._msg_send(buf.getvalue()):
-                self.publish_error = True
+            if self.dry_run:
+                print_info(self.logger, 'Printing the content of the APEL summary instead of sending it to APEL')
+                print(buf.getvalue())
+            else:
+                if not self._msg_send(buf.getvalue()):
+                    self.publish_error = True
             buf.close()
             del buf
 
@@ -636,8 +634,12 @@ class APELAMSDirectSender(object):
             buf.write(APELSyncRecord.header())
             buf.write(APELSyncRecord.join_str().join(map(lambda s: s.get_record(), syncs[x:x + self.batchsize])))
             buf.write(APELSyncRecord.footer())
-            if not self._msg_send(buf.getvalue()):
-                self.publish_error = True
+            if self.dry_run:
+                print_info(self.logger, 'Printing the content of the APEL Sync instead of sending it to APEL')
+                print(buf.getvalue())
+            else:
+                if not self._msg_send(buf.getvalue()):
+                    self.publish_error = True
             buf.close()
             del buf
 
@@ -647,7 +649,8 @@ class APELAMSDirectSender(object):
             self.logger.error('Failed to publish records to APEL AMS %s. See previous messages for details.',
                               self.conf['targethost'])
             return False
-        self.logger.debug('AMS direct records sending to %s has finished.', self.conf['targeturl'])
+        if not self.dry_run:
+            self.logger.debug('AMS direct records sending to %s has finished.', self.conf['targeturl'])
         return True
 
 
@@ -702,12 +705,11 @@ class UsageRecord(JobAccountingRecord):
                       '<tr:StartTime>{tstart}</tr:StartTime><tr:EndTime>{tend}</tr:EndTime></tr:FileUpload>'
     }
 
-    def __init__(self, aar, localid_prefix=None, vomsless_vo=None, extra_vogroups=None):
+    def __init__(self, aar, localid_prefix=None, extra_vogroups=None):
         """Create XML representation of UsageRecord"""
         JobAccountingRecord.__init__(self, aar)
         self.logger = logging.getLogger('ARC.Accounting.UR')
         self.localid_prefix = localid_prefix
-        self.vomsless_vo = vomsless_vo
         self.extra_vogroups = extra_vogroups
         if self.extra_vogroups is None:
             self.extra_vogroups = []
@@ -746,12 +748,11 @@ class UsageRecord(JobAccountingRecord):
         wlcgvo = self.aar.wlcgvo()
         voissuer = ''  # no issuer info in current implementation, is this mandatory?
         if not wlcgvo:
-            # arc.conf: vomsless VO
-            if self.vomsless_vo is not None:
-                vomsless_data = self.vomsless_vo.split('#')
-                wlcgvo = vomsless_data[0]
-                if len(vomsless_data) > 1:
-                    voissuer = vomsless_data[1]
+            return vomsxml
+        vomsless_data = wlcgvo.split('#')
+        wlcgvo = vomsless_data[0]
+        if len(vomsless_data) > 1:
+            voissuer = vomsless_data[1]
         if not wlcgvo:
             return vomsxml
         self.log += ' (VO {0})'.format(wlcgvo)
@@ -925,11 +926,10 @@ class ComputeAccountingRecord(JobAccountingRecord):
                    '{localuser}{localgroup}{groups}</UserIdentity>',
     }
 
-    def __init__(self, aar, vomsless_vo=None, extra_vogroups=None, gocdb_name=None):
+    def __init__(self, aar, extra_vogroups=None, gocdb_name=None):
         """Create XML representation of Compute Accounting Record"""
         JobAccountingRecord.__init__(self, aar)
         self.logger = logging.getLogger('ARC.Accounting.CAR')
-        self.vomsless_vo = vomsless_vo
         self.extra_vogroups = extra_vogroups
         if self.extra_vogroups is None:
             self.extra_vogroups = []
@@ -962,11 +962,6 @@ class ComputeAccountingRecord(JobAccountingRecord):
         xml = ''
         # Group (optional recommended) - the effective User VO of the user running the job
         wlcgvo = self.aar.wlcgvo()
-        if not wlcgvo:
-            # arc.conf: vomsless VO
-            if self.vomsless_vo is not None:
-                vomsless_data = self.vomsless_vo.split('#')
-                wlcgvo = vomsless_data[0]
         if wlcgvo:
             xml += '<Group>{0}</Group>'.format(wlcgvo)
             self.log += ' (VO {0})'.format(wlcgvo)
