@@ -119,7 +119,7 @@ class JobsControl(ComponentControl):
 
     def __parse_job_attrs(self, jobid, attrtype='local'):
         _KEYVALUE_RE = re.compile(r'^([^=]+)=(.*)$')
-        attr_file = '{0}/job.{1}.{2}'.format(self.control_dir, jobid, attrtype)
+        attr_file = control_path(self.control_dir, jobid, attrtype)
         job_attrs = {}
         if os.path.exists(attr_file):
             with open(attr_file, 'r') as attr_f:
@@ -240,7 +240,7 @@ class JobsControl(ComponentControl):
             sys.stdout.flush()
 
     def job_script(self, args):
-        error_log = '{0}/job.{1}.errors'.format(self.control_dir, args.jobid)
+        error_log = control_path(self.control_dir, args.jobid, 'errors')
         if os.path.exists(error_log):
             el_f = open(error_log, 'r')
             print_line = False
@@ -303,9 +303,12 @@ class JobsControl(ComponentControl):
             self.logger.error('Failed to find log file: %s', log)
 
     def job_log(self, args):
-        error_log = '{0}/job.{1}.errors'.format(self.control_dir, args.jobid)
+        error_log = control_path(self.control_dir, args.jobid, 'errors')
         if os.path.exists(error_log):
-            self.__follow_log(error_log, args.follow, process_function=self.__cut_jobscript)
+            if args.raw:
+                self.__follow_log(error_log, args.follow)
+            else:
+                self.__follow_log(error_log, args.follow, process_function=self.__cut_jobscript)
         else:
             self.__get_jobs()
             self.__job_exists(args.jobid)
@@ -319,15 +322,14 @@ class JobsControl(ComponentControl):
         if self.arcconfig.check_blocks('arex/ws'):
             arexws_log = self.__get_config_value('arex/ws', 'logfile', '/var/log/arc/ws-interface.log')
             self._service_log_print(arexws_log, args.jobid)
-        # GridFTP interface logs
-        if self.arcconfig.check_blocks('gridftpd'):
-            gridftpd_log = self.__get_config_value('gridftpd', 'logfile', '/var/log/arc/gridftpd.log')
-            self._service_log_print(gridftpd_log, args.jobid)
         # A-REX jobs log
         arexjob_log = self.__get_config_value('a-rex', 'joblog')
         if arexjob_log is not None:
             self._service_log_print(arexjob_log, args.jobid)
 
+    def job_path(self, args):
+        print(control_path(self.control_dir, args.jobid, ''))
+
     def jobinfo(self, args):
         self.__get_jobs()
         self.__job_exists(args.jobid)
@@ -338,6 +340,8 @@ class JobsControl(ComponentControl):
               'Modified\t: {modified}'.format(**self.jobs[args.jobid]))
 
     def job_stdout(self, args):
+        self.__get_jobs()
+        self.__job_exists(args.jobid)
         job_grami = self.__parse_job_attrs(args.jobid, 'grami')
         if not 'joboption_stdout' in job_grami:
             self.logger.error('Cannot find executable stdout location for job %s', args.jobid)
@@ -345,6 +349,8 @@ class JobsControl(ComponentControl):
         self.__follow_log(job_grami['joboption_stdout'], args.follow)
 
     def job_stderr(self, args):
+        self.__get_jobs()
+        self.__job_exists(args.jobid)
         job_grami = self.__parse_job_attrs(args.jobid, 'grami')
         if not 'joboption_stderr' in job_grami:
             self.logger.error('Cannot find executable stderr location for job %s', args.jobid)
@@ -352,6 +358,8 @@ class JobsControl(ComponentControl):
         self.__follow_log(job_grami['joboption_stderr'], args.follow)
 
     def job_getattr(self, args):
+        self.__get_jobs()
+        self.__job_exists(args.jobid)
         job_attrs = self.__parse_job_attrs(args.jobid)
         if args.attr:
             if args.attr in job_attrs:
@@ -426,6 +434,7 @@ class JobsControl(ComponentControl):
 
     def control(self, args):
         self.cache_ttl = args.cachettl
+        canonicalize_args_jobid(args)
         if args.action == 'list':
             self.list(args)
         elif args.action == 'killall':
@@ -451,6 +460,8 @@ class JobsControl(ComponentControl):
             self.job_stderr(args)
         elif args.action == 'attr':
             self.job_getattr(args)
+        elif args.action == 'path':
+            self.job_path(args)
         elif args.action == 'stats':
             self.job_stats(args)
         elif args.action == 'accounting' and AccountingControl is not None:
@@ -497,6 +508,8 @@ class JobsControl(ComponentControl):
         jobs_log = jobs_actions.add_parser('log', help='Display job log')
         jobs_log.add_argument('jobid', help='Job ID').completer = complete_job_id
         jobs_log.add_argument('-f', '--follow', help='Follow the job log output', action='store_true')
+        jobs_log.add_argument('-r', '--raw', help='Show raw logfile content as it is (including jobscript)',
+                              action='store_true')
         jobs_log.add_argument('-s', '--service', help='Show ARC CE logs containing the jobID instead of job log',
                               action='store_true')
 
@@ -511,10 +524,13 @@ class JobsControl(ComponentControl):
         jobs_stderr.add_argument('jobid', help='Job ID').completer = complete_job_id
         jobs_stderr.add_argument('-f', '--follow', help='Follow the job log output', action='store_true')
 
-        jobs_attr = jobs_actions.add_parser('attr', help='Get ')
+        jobs_attr = jobs_actions.add_parser('attr', help='Get job attribute')
         jobs_attr.add_argument('jobid', help='Job ID').completer = complete_job_id
         jobs_attr.add_argument('attr', help='Attribute name', nargs='?')
 
+        jobs_path = jobs_actions.add_parser('path', help='Print control directory path for a job')
+        jobs_path.add_argument('jobid', help='Job ID').completer = complete_job_id
+
         jobs_kill = jobs_actions.add_parser('kill', help='Cancel job')
         jobs_kill.add_argument('jobid', nargs='+', help='Job ID').completer = complete_job_id
 
